import sys
sys.path[0:0] = ['..', '../../m5/util/config']

from m5config import *

SYSTEMDIR = '/z/m5/system/stable'
BINDIR = SYSTEMDIR + '/binaries'
DISKDIR = SYSTEMDIR + '/disks'

root = ConfigNode('Universe')

system = Tru64System('system', root)
system.kernel_code = BINDIR + '/vmunix'
system.pal_code = BINDIR + '/osfpal'
system.console_code = BINDIR + '/console'
system.binned_fns = ''

tlaser = Turbolaser('tlaser', system)

mmu = MemoryController('mmu', system)
system.mem_ctl = mmu

cpu = SimpleCPU('cpu', system, itb=AlphaITB('itlb'), dtb=AlphaDTB('dtlb'),
                mem=mmu, system=system)

ram = PhysicalMemory('ram', system, addr=0, size=128*M, mmu=mmu)
system.physmem = ram

intrctrl = IntrControl('intrctrl', system, cpu=cpu)
tlaser.intrctrl = intrctrl

image0 = CowDiskImage('image0', system, image_file=DISKDIR+'/tru64_image.cow',
                      child=RawDiskImage('rdi',
                                         image_file=DISKDIR+'/tru64_image',
                                         read_only=true),
                      read_only=True)
image1 = RawDiskImage('image1', system, image_file=DISKDIR+'/exit_image',
                      read_only=True)

simple_disk = SimpleDisk('simple_disk', disk=image0, physmem=ram)

sim_console = SimConsole('sim_console', system,
                         listener = ConsoleListener('listener', port=3456),
                         intr_control = intrctrl)
tlaser.cons = sim_console

clock = TlaserClock('clock', frequency=1200, intr_control=intrctrl)
tlaser.clock = clock

tlaser.ipi = TlaserIpi('ipi', addr=0xff8e000000, mmu=mmu)
tlaser.mbox = TlaserMBox('mbox', physmem=ram)

TlaserMC146818('rtc', tlaser, addr=0xffb0000000, mmu=mmu, clock=clock)
TlaserUart('uart', tlaser, addr=0xffa0000000, mmu=mmu, console=sim_console)
TlaserSerial('sernum', tlaser, addr=0xffc7000000, mmu=mmu)
TlaserPcia('pcia', tlaser, addr=0xc780000000, mmu=mmu)
TlaserNode('node0', tlaser, tlaser=tlaser, number=0, node_type='SingleProc4M',
           addr=0xff88000000, mmu=mmu)
TlaserNode('node1', tlaser, tlaser=tlaser, number=4, node_type='MS7CC',
           addr=0xff89000000, mmu=mmu)
TlaserNode('node2', tlaser, tlaser=tlaser, number=8, node_type='KFTIA',
           addr=0xff8a000000, mmu=mmu)
AlphaConsole('console', tlaser, clock=clock, disk=simple_disk,
             sim_console=sim_console, system=system, cpu=cpu,
             addr=0x8000a00000, mmu=mmu)

dma_engine = DmaEngine('dma_engine', tlaser, physmem=ram)

disk0 = ScsiDisk('disk0', image=image0, target=1, serial='00Q8127609LSM0')
disk1 = ScsiDisk('disk1', image=image1, target=2, serial='00Q8127609LSM1')

scsi_pci_dev = TlaserPciDev('dev', VendorID=0x1291, DeviceID=0x1291,
                            SubClassCode=0x00, ClassCode=0x01, ProgIF=0x00,
                            BAR0=0x0, BAR1=0x0, BAR2=0x0, BAR3=0x0,
                            BAR4=0x0, BAR5=0x0,
                            InterruptLine=0x09, InterruptPin=0x01,
                            addr=0xc700040000, mmu=mmu)

tlaser.scsi = ScsiController('scsi', intr_ctrl=intrctrl, engine=dma_engine,
                             pci_dev=scsi_pci_dev,
                             disks=[disk0, disk1],
                             addr=0xc500000000, mmu=mmu)

ether_pci_dev = TlaserPciDev('dev', VendorID=0x1291, DeviceID=0x1292,
                             SubClassCode=0x00, ClassCode=0x02, ProgIF=0x00,
                             BAR0=0x00010000, BAR1=0x00010000, BAR2=0x00010000,
                             BAR3=0x00010000, BAR4=0x00010000, BAR5=0x00010000,
                             InterruptLine=0x09, InterruptPin=0x02,
                             addr=0xc700080000, mmu=mmu)

etherdev = EtherDev('etherdev', intr_ctrl=intrctrl, physmem=ram,
                    pci_dev=ether_pci_dev,
                    addr=0xc500010000, mmu=mmu)
tlaser.ethernet = etherdev

etherint = EtherDevInt('etherint', tlaser, device=etherdev)
etherdump = EtherDump('etherdump', root, file='/z/tmp/ethertrace')
ethertap = EtherTap('ethertap', tlaser, peer=etherint, packet_dump=etherdump)

instantiate(root)
