#! /usr/bin/env perl
use Cwd;

$usage = '    Usage: perl do-tests.pl [-D|-F|-O] [-T tests] [-d] <build/ path>
                            [-t] <m5-test/ path> [-m] <flags> [-v]
                            [-C] [-B builds] [-R] [-U]

          OPTIONS
          -D  make and test ONLY debug
          -F  make and test ONLY fast
          -O  make and test ONLY opt
          ***default: opts only

          -T name of test set (daily, weekly, monthly, interactive) or
             name of single test.  "all" does union of all test sets.
             example: "-T daily".  default: interactive
          -B name of build or file listing the builds you want to test.
             default: BUILDS

          ***directory paths must be absolute
          -d absolute path of build directory, e.g. /z/foo/build
             default is ../m5/build
          -t absolute path of test directory, default is cwd
          -l path to log directory, default is "log"

          -m flags you want to send to "make", for example if you want
             make -j N, you say -m "-j -p"
          -j parallel make: -j N is short for -m "-j N"
          -v verbose output - output of diff-out will go to stdout
             output of make will go to stdout if there was an error
          -C make clean

          ***these switches only used by regression script
          ***usually no need for end user to use
          -R turn this switch if it is a cron call for
             regression testing rather than self-test
          -U m5-tests updated.  used only by regression.pl

   **TO SEE THIS WHENEVER YOU WANT, TYPE: perl do-tests.pl --help
';

print "\nStarting do-tests.pl.....\n";

###################
#Parsing arguments#
###################
foreach (@ARGV) {
  if($_ eq "--help") {
    print "$usage\n";
    exit;
  }
}
parse_args();

#########################
#Change to m5-test dir  #
#########################
if($opt_t) {
  chdir("$test_dir") || mail_and_die("Cannot change directory to $test_dir!: $!\n");
}

################################################################################
#Which builds are we going to try and make?  Default is from file called BUILDS#
#Otherwise, can be specified by user with flag -B                              #
################################################################################
if (-r $builds_name) {
  open (BUILDS, $builds_name) || mail_and_die ("Could not open file $builds_file: $!\n");
  @builds = <BUILDS>;
  chomp (@builds);
} else {
  @builds = ($builds_name);
}

$output_file = "Output.$$";

#######################################################################
#Go through each BUILD (ALPHA, KERNEL, etc) and each test (opt, debug)#
#######################################################################
$num_errors=0; #records total number of errors in run of do-tests
$error = 0;  #records whether there is error at all (so that at end, act accordingly)
foreach $build (@builds) {
  foreach $test (@tests) {
    #if the build directory isn't there, try and do setup.
    if( !(chdir "$build_dir/$build")) {
      chdir "$source_dir/setup" || mail_and_die("Cannot chdir into $source_dir/setup:\n$!");
      $ENV{PWD} = `pwd`;
      system("./setup $build") && mail_and_die("Cannot execute setup $build:\n$!\n");
      chdir "$build_dir/$build" || mail_and_die("Something really's wrong!, directory $build_dir/$build:\n$!\n");
    }

    #only make clean if it's a regression && something has been updated OR if -C flag
    if (($opt_R && $opt_U) || ($opt_C)) {
      print "making clean in " . cwd() . "\n";
      system("make clean") && mail_and_die("Could not make clean in $build!: $!\n");
    }

    print "\nMaking $build/m5.$test\n";
    # Tee make output to stdout in real time if -v
    $make_out = "MAKE.$$";
    $redir = $opt_v ? "2>&1 | tee $make_out" : "> $make_out 2>&1";
    $make_args = $opt_m;
    $make_args .= " -j $opt_j" if $opt_j;
    $cmd = "make $make_args m5.$test $redir";
    if (system($cmd)) {
      $error = 1;
      $num_errors++;
      open MAKE,"$make_out" || mail_and_die("Can't open $make_out!:\n$!\n");
      @tmp = <MAKE>;
      push(@args, "****ERROR NUMBER $num_errors****\n");
      push(@args, "**make m5.$test failed for $build!: $!\nMake output:\n\n");
      push(@args, @tmp);
      if ($opt_v) {
	print "****ERROR NUMBER $num_errors****\n";
	print "**make m5.$test failed for $build!\n";
      }
      print "\n\nMoving on to next binary...\n";
      next;
    } else {
	unlink($make_out);
    }

    $simulator = "$build_dir/$build/m5.$test";
    chdir "$test_dir" || mail_and_die("Cannot cd into $test_dir: $!\n");

    # We only want to use the 'base' part of the build name to find
    # the appropriate set of tests; for example, if the build is
    # KERNEL.TRACE, we still want to run the tests from
    # KERNEL-interactive-tests.

    ($build_base, $dummy) = split('\.', $build);

    if ($type eq "all") {
      #the following complicated stuff is to get a union set of all build-*-tests
      @test_files = glob("$build_base-*-tests");
      if (! @test_files) {
	# no tests for this build... just skip it
	print "no files matching $build_base-*-tests, no tests will be run\n";
	next;
      }
      @test_names = ();
      foreach (@test_files) {
	open TESTS, "$_";
	@tmp = <TESTS>;
	chomp(@tmp);
	push(@test_names, @tmp);	
      }
      @test_names = sort(@test_names);
      @temp = @test_names;
      $last = "";
      $pos = 0;
      foreach (@temp) {
	if ($_ eq $last) {
	  splice (@test_names, $pos, 1);
	  $pos--;
	}
	$pos++;
	$last = $_;
      }
    } elsif (-r "$build_base-$type-tests") {
      open(TESTS, "$build_base-$type-tests") || mail_and_die("Can't open $build_base-$type-tests: $!\n");
      @test_names = <TESTS>;
      chomp(@test_names);
    } elsif (-d $type) {
      # arg could also be just the name of a single test
      @test_names = ($type);
    } else {
      print "no file $build_base-$type-tests and no directory $type: no tests will be run\n";
      next;  #if there's no tests for this build, then just go on
    }

    foreach $test_name (@test_names) {	
      print "\n**** Running $test_name on $simulator ****\n";
      $refout = "$test_name/simout.ref";

      if (! (-f $simulator && -x _)) {
	mail_and_die ("Error: simulator $simulator not found or not executable.\n");
      }
      if (! (-d $test_name && -r _)) {
	mail_and_die ("Error: test directory $test_name not found.\n");
      }
      if (! (-f $refout && -r _)) {
	mail_and_die ("Error: reference output $refout not found.\n");
      }

      if ( -e $log_dir) {
	if( -d $log_dir) {
	  #if log exists and is a directory, then make sure the sub dir's exist, else
	  #make them
	  chdir $log_dir || mail_and_die("Can't chdir into $log_dir!:\n$!\n");
	  if (!( -d "$test_name")) {
	    mkdir "$test_name" || mail_and_die("Can't mkdir $test_name!:\n$!\n");
	  }
	} else {
	  mail_and_die("please reserve the name \"$log_dir\" for a log directory.\n");
	} #if-else log is a dir
      } else {
	mkdir $log_dir || mail_and_die("Can't mkdir $log_dir!:\n$!\n");
	chdir $log_dir || mail_and_die("Can't chdir into $log_dir!:\n$!\n");
	mkdir "$test_name" || mail_and_die("Can't mkdir $test_name!:\n$!\n");
      } #if-else log exists

      chdir "$test_dir" || mail_and_die("Can't chdir into $test_dir!:\n$!\n");
      $newout = "$log_dir/$test_name/simout.ref";
      $command = "$simulator $test_name/run.ini --Universe:output_file=stdout";

      $pipe=0;
      if (system("$command > $newout 2>&1 < /dev/null")) {
	open SIMOUT, $newout || mail_and_die("Can't open file $newout!:\n$!\n");
	while (<SIMOUT>) {
	  push(@tmp, $_);
	  if (/gzip: stdout: Broken pipe/) {
	    $pipe=1;
	    last;
	  }
	}
	if ($pipe==0) {
	  push(@args,"****ERROR NUMBER $num_errors****\n");
	  push(@args,"**$command failed! \nAbnormal program termination!!!! Output below\n\n");
	  push(@args,@tmp);
	  print "**$command failed!\n Abnormal program termination!!!! Moving on to next test\n";
	  $error=1;
	  $num_errors++;
	  next;
	}
      }
      $cmd = "./diff-out -p $refout $newout ";
      $tmpfile = "tmp.$$";
      if($opt_v) {
	print "\n----------Diff-Out for $build/$test_name---------\n\n";
	$cmd = $cmd . "2>&1 | tee $tmpfile";
      } else {
	$cmd = $cmd . "> $tmpfile 2>&1";
      }
      system("echo \"\n******Diff-Out for $simulator $test_name ******\n\" > $tmpfile");
      system("$cmd") && mail_and_die("Diff-out failed! See file $tmpfile for details\n");

      open(OUTPUT,"$tmpfile") || mail_and_die("Cannot open $tmpfile: $!\n");

      $flag=0; #only want to increment num_errors on first error in a single test
      while(<OUTPUT>) {
	if(/^Maximum error magnitude:/) {
	  @mag = split(/[+%]/,$_);
	  $mag = $mag[1];
	  if($mag > 0) {
	    if($flag==0) {
	      $num_errors++;
	      $flag=1;
	      push(@args,"\n\n****ERROR NUMBER $num_errors\n");
	    }
	    push(@args,"**Error in test $test_name for $simulator: Non-zero error magnitude\n");
	    $error = 1;
	  }
	}
	if(/^Missing /) {
	  if($flag==0) {
	    $flag=1;
	    $num_errors++;
	    push(@args,"\n\n****ERROR NUMBER $num_errors\n");
	  }
	  push(@args,"**Error in test $test_name for $simulator: Missing ref stat\n");
	  $error=1;
	}
	if(/^Found /) {	 
	  if($flag==0) {
	    $flag=1;
	    $num_errors++;
	    push(@args,"\n\n****ERROR NUMBER $num_errors\n");
	  }
	  push(@args,"**Error in test $test_name for $simulator: Found extra stats\n");
	  $error=1;
	}
	if ($flag) {
	  push(@args,$_);
	}
      } #while <OUTPUT>
      system("echo '>>>>>>> Diff output for $simulator $test_name <<<<<<<' >> $output_file");
      system("cat $tmpfile >> $output_file") && mail_and_die("Could not append diff out for $build/$test_name to $output_file!\n");
    } #foreach test_names
  } #foreach tests
} #foreach builds

close OUTPUT;
unlink($tmpfile);
#put this here so you do everything you can and collect all the
#errors before reporting them
if($error) {
  $num_errors--;
  mail_and_die(@args);
}

sub parse_args {
  #redundant, but do it here in case you die, you have a home to go to
  $test_dir = cwd();
  chomp($test_dir);


  use Getopt::Std;
  getopts("DFOT:d:t:l:j:m:vCB:RU") || mail_and_die("Fix Syntax:\n$usage\n");

  if (!($opt_D || $opt_F || $opt_O)) {
    @tests = "opt";
  }
  if ($opt_D) {
    push(@tests, "debug");
  }

  if ($opt_F) {
    push(@tests, "fast");
  }

  if ($opt_O) {
    push(@tests, "opt");
  }

  if ($opt_T) {
    $type = $opt_T;
  } else {
    if ($opt_R) {
      $type = "daily";
    } else {
      $type = "interactive";
    }
  }

  if (!$opt_d) {
    $opt_d = "..";
    $build_dir = build_directory($opt_d,"m5/build");
    $source_dir = build_directory($opt_d,"m5");
  } else {
    $build_dir = $opt_d;
    $source_dir = $build_dir . "/src";
  }

  if($opt_t) {
    $test_dir = $opt_t;
  }

  $log_dir = $opt_l || "log";

  if($opt_B) {
    $builds_name = $opt_B;
  } else {
    $builds_name = "BUILDS";
  }
} #sub parse_args

sub mail_and_die {
  $num_errors++;
  chdir "$test_dir" || die("Cannot change into directory $test_dir!:\n$!\n");

  #if it's a regression test, mail errors to m5-dev, else print to stdout
  if($opt_R) {
    open MAIL, '|mail -s "REGRESSION" m5-dev@eecs.umich.edu';
    print MAIL "**There were $num_errors errors in this run!  grep for **** to see all the separate errors\n";
    foreach (@_) {
      print MAIL;
    }
    print MAIL "\nregression test unsuccessful!!";
    close MAIL;
    exit;
  } else {
    $errfile = "ERRORS.$$";
    print "!!!!! There were $num_errors errors in this run!  See file \"$errfile\" for details.\n  Each new error line is headed by ****\n";
    open ERRORS, ">$errfile" || die("cannot open $errfile\n");
    foreach (@_) {
      print ERRORS;
    }
    exit;
  }
}

sub build_directory {
  my ($base, $dir) = @_;
  if ($base =~ /^\//) {
    return $base . "/" . $dir;
  }
  return cwd() . "/" . $base . "/" . $dir;
}


chdir "$test_dir" || die("Cannot change into directory $test_dir!:\n$!\n");

print "finished do-tests successfully!\n";
