#! /usr/bin/env perl
$usage = '    Usage: perl regression.pl [-D|-O] [-T] <test type>
                            [-d] <m5/ path> [-t] <m5-test/ path>
                            [-p] <parent path> [-R] [-v]

          OPTIONS
          -D  make and test ONLY debugs
          -O  make and test ONLY opts
          ***default: opts

          -T type of test: daily, weekly, monthly, interactive, all

          ***directory paths must be absolute
	  -d absolute path of build directory

	  -t absolute path of test directory
  	  default: test = `pwd`, dir = ../m5/build

          -v verbose
';

#setup the call to do-tests.pl
$command = "perl ./do-tests.pl ";
foreach (@ARGV) {
  $command = $command . "$_ ";
}

#always include -R  and -C since this is a regression test
$command = $command . "-R -C -m \"-j4\" ";

use Getopt::Std;
getopts("DOTd:t:v") || mail_and_die("  Fix Syntax:\n$usage\n");

#default test_dir is /z/m5/regression/m5-test
if (!$opt_t) {
  $opt_t = "/z/m5/regression/m5-test";
  $command = $command . "-t $opt_t ";
}
$test_dir = $opt_t;
if (opt_v) {
    print "test directory is $test_dir\n";
}

#default build directory is /z/m5/regression/m5/build
if (!$opt_d) {
  $opt_d = "/z/m5/regression/m5/build";
  $command = $command . "-d $opt_d ";
}
$build_dir = $opt_d;
if (opt_v) {
    print "build directory is $build_dir\n";
}

#if setup hasn't been run yet (i.e. no /src symlink) try and do it now
if (!(-e "$build_dir" . "/src")) {
    if (opt_v) {
        print "setup not done yet - no build/src dir\n";
    }
  @m5 = split(/\//, $build_dir);
  pop @m5;
  $m5 = join('/', @m5);
    if (opt_v) {
        print "m5 directory to setup is $m5\n";
    }
  chdir "$m5/setup" || mail_and_die("Can't chdir into $m5/setup to do setup!  Please run setup yourself\n");
  $ENV{PWD} = `pwd`;
    if (opt_v) {
        print "$ENV{PWD} is the setup dir\n";
    }
  system ("./setup ALPHA") && mail_and_die("default setup failed, please run setup yourself\n");
}

$m5_dir = $build_dir . "/src";

chdir "$test_dir" || mail_and_die("Could not cd into $test_dir!: $!\n");
system("bk pull >& pull") && mail_and_die("bk pull for test dir $test_dir failed!: $!\n");
open PULL, "pull" || mail_and_die("Could not open file pull: $!\n");
$nothing = 0;
while (<PULL>) {
  if (/Nothing to pull/) {
    $nothing=1;
  }
}
if (!$nothing) {
  $command = $command . "-U ";
}

chdir "$m5_dir" || mail_and_die("Could not cd into $m5_dir!: $!\n");
system("bk pull >& pull") && mail_and_die("bk pull for m5 dir $m5_dir  failed!: $!\n");
open  PULL, "pull" || mail_and_die("Could not open file pull: $!\n");
$nothing = 0;
while (<PULL>) {
  if (/Nothing to pull/) {
    $nothing=1;
  }
}
if (!$nothing) {
  $command = $command . "-U ";
}

chdir "$test_dir" || mail_and_die("Could not cd into $test_dir!: $!\n");
print "executing $command\n";
system("$command") && mail_and_die("Cannot execute $command!:\n$!");

sub mail_and_die {
  open MAIL, '|mail -s "REGRESSION" hsul@eecs.umich.edu'; 
  foreach (@_) {
    print MAIL;
  }
  print MAIL "\nregression test unsuccessful!!";
  close MAIL;
  print "regression test unsuccessful!\n";
  die;
}

print "finished regression successfully!\n";
