/* $Id: s.insn_fifo.hh 1.10 04/02/02 18:14:09-08:00 stever@isabel.reinhardt.house $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#ifndef __INSN_FIFO_HH__
#define __INSN_FIFO_HH__

#include "base/fifo_buffer.hh"
#include "cpu/full_cpu/dyn_inst.hh"
#include "cpu/full_cpu/smt.hh"


class InstructionFifo {
  private:
    struct InstRecord {
	DynInst *inst;
	unsigned    thread;

	// constructor
	InstRecord(void) : inst(0) {}
	InstRecord(DynInst *i)
	    : inst(i), thread(i->thread_number) {}

	void squash(void) {
	    inst->squash();
	    delete inst;
	    inst = 0;
	}

	void dump(void) {
	    assert(inst != 0);
	    inst->dump();
	}
    };


    FifoBuffer<InstRecord> *instructions;

    typedef FifoBuffer<InstRecord>::iterator iterator;

    unsigned insts[SMT_MAX_THREADS];

    unsigned size;

  public:
    InstructionFifo(unsigned size);

    unsigned add(DynInst *inst);

    unsigned count(void) { return instructions->count(); }
    unsigned count(unsigned t) { return insts[t]; }

    DynInst * peek(void);
    DynInst * remove(unsigned *thread = 0);

    unsigned squash(void);
    unsigned squash(unsigned t);

    void dump(void);
    void dump(unsigned thread);

    ~InstructionFifo(void) { delete instructions; }
};


#endif
