/* $Id$ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#include <cassert>
#include <iomanip>
#include <iostream>

#include "base/cprintf.hh"
#include "base/python.hh"

using namespace std;

template <int N>
class Spaces
{
  private:
    char spaces[N + 1];

  public:
    Spaces()
    {
	memset(spaces, ' ', N);
	spaces[N] = '\0';
    }

    const char *operator[](int number) const
    {
	assert(number >= 0 && number <= N);
	return &spaces[N - number];
    }
    int num() const { return N; }
};

std::string
Python::c2py(double val)
{
    if (isnan(val))
	return "\'NaN\'";
    if (isinf(val))
	return "\'Inf\'";
    return to_string(val);
}


Python::Python(ostream &s)
    : stream(s), level(0), endline(false), close(0), comma(false)
{
}

Python::~Python()
{
}

Spaces<256> spcs;

void
Python::name(const string &name)
{
    next();
    stream << name << "(";
    ++level;
    comma = false;
}

void
Python::nameEnd()
{
    close = ')';
    comma = false;
    --level;
    next();
}

void
Python::tuple(const string &name)
{
    next();
    ++level;
    comma = false;
    if (!name.empty())
	stream << name << "=";
    stream << "(";
}

void
Python::tupleEnd()
{
    close = ')';
    comma = false;
    --level;
    next();
}

void
Python::list()
{
    next();
    ++level;
    comma = false;
    stream << "[";
}

void
Python::listEnd()
{
    close = ']';
    comma = false;
    --level;
    next();
}

void
Python::next()
{
    if (comma && level)
	stream << ',';

    if (endline)
	ccprintf(stream, "\n%s", spcs[level * 4]);

    if (close)
	stream << close;

    endline = false;
    comma = true;
    close = 0;
}

void
Python::newline()
{
    endline = true;
}

void
Python::arg(const string &val)
{
    next();
    stream << val;
}

void
Python::kwarg(const string &name, const string &val)
{
    next();
    stream << name << "=" << val;
}
