/* $Id: s.etherpkt.hh 1.13 04/02/21 10:43:36-05:00 binkertn@ziff.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

/* @file
 * Reference counted class containing ethernet packet data
 */

#ifndef __ETHERPKT_HH__
#define __ETHERPKT_HH__

#include <iosfwd>
#include <memory>

#include "sim/host.hh"
#include "base/refcnt.hh"

class Checkpoint;

/*
 * Reference counted class containing ethernet packet data
 */
class EtherPacket : public RefCounted
{
  public:
    uint8_t *data;
    int length;

  public:
    EtherPacket() : data(NULL), length(0) {}
    EtherPacket(std::auto_ptr<uint8_t> d, int l)
	: data(d.release()), length(l) {}
    ~EtherPacket() { if (data) delete [] data; }

  public:
    bool IsUnicast() { return data[0] == 0x00; }
    bool IsMulticast() { return data[0] == 0x01; }
    bool IsBroadcast() { return data[0] == 0xff; }

    void serialize(std::ostream &os);
    void unserialize(Checkpoint *cp, const std::string &section);
};

typedef RefCountingPtr<EtherPacket> PacketPtr;

#endif // __ETHERPKT_HH__
