/* $Id: s.scsi.hh 1.15 03/10/16 17:56:15-04:00 ehallnor@zizzer.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

/* @file
 * SCSI protocol structures and variable definitions
 */

#ifndef __SCSI_HH__
#define __SCSI_HH__

#include "sim/host.hh"

// SCSI commands.
//
// Not all of these are supported in this device. Currently, only
// the commands used by IRIX are implemented.
#define CMD_TST_UNIT_RDY	0x0
#define CMD_REQ_SENSE		0x3
#define CMD_FORMAT		0x4
#define CMD_ADD_DEFECTS		0x7
#define CMD_READ_6              0x8
#define CMD_WRITE_6             0xa
#define CMD_INQUIRY		0x12
#define CMD_MODE_SELECT		0x15
#define CMD_MODE_SENSE		0x1a
#define CMD_STARTUNIT		0x1b
#define CMD_SEND_DIAG		0x1d
#define CMD_PREVREM		0x1e
#define CMD_READCAPACITY	0x25
#define CMD_READ		0x28
#define CMD_WRITE		0x2a
#define CMD_SEEK		0x2b
#define CMD_ERASE		0x2c
#define CMD_RDEFECTS		0x37

#define CMD_NONE                0xff // Flag meaning no SCSI command

const char *ScsiOpcode(int index);
// Taken from /usr/include/sys/dksc.h

// The data structure of the mode sense command
struct ModeSenseData
{
    uint8_t  sense_len;
    uint8_t  mediatype;
    uint8_t  wprot:1, reserv0:2, dpofua:1, reserv1:4;
    uint8_t  bd_len;
    uint8_t  block_descrip[8];
    // Don't need this:    union   dk_pages dk_pages;
};


// from io/cam/scsi_all.h
struct AllInqData
{
    uint8_t	dtype	: 5,	// Peripheral device type.	[0]
		pqual	: 3;	// Peripheral qualifier.
    uint8_t	dmodify	: 7,	// Device type modifier.	[1]
		rmb	: 1;	// Removable media.
    uint8_t	ansi	: 3,	// ANSI version.		[2]
		ecma	: 3,	// ECMA version.
		iso	: 2;	// ISO version.
    uint8_t	rdf	: 4,	// Response data format.	[3]
		RSVR0	: 2,	// Reserved.
		trmiop	: 1,	// Terminate I/O process
		aenc	: 1;	// Async Notification of events
    uint8_t	addlen;		// Additional length.		[4]
    uint8_t	RSVR1	: 8;	// Reserved.			[5]
    uint8_t	RSVR2	: 8;	// Reserved.			[6]
    uint8_t	sftre	: 1,	// Soft reset 1 = yes		[7]
		cmdque	: 1,	// Command queuing
		RSVR3	: 1,	// Reserved bit
		linked	: 1,	// Linked command support
		sync	: 1,	// Synchronous data transfers
		wbus16	: 1, 	// support of 16 bit transfers
		wbus32	: 1,	// support of 32 bit transfers
		reladdr : 1;	// Relative addressing support
    uint8_t	vid[8];		// Vendor ID.		     [8-15]
    uint8_t	pid[16];	// Product ID.		    [16-31]
    uint8_t	revlevel[4];	// Revision level.	    [32-35]
};

struct EvpdInqData
{
    uint8_t	dtype		: 5,	// Peripheral device type.	[0]
		pqual		: 3;	// Peripheral qualifier.
    uint8_t	page_code	: 8;
    uint8_t			: 8;
    uint8_t	page_length	: 8;
    uint8_t	page_codes[251];
};

struct DirReadCapData
{
    uint8_t lbn3;		// MSB of number of logical blocks
    uint8_t lbn2;		// MID HIGH of number of logical blocks
    uint8_t lbn1;		// MID LOW of number of logical blocks
    uint8_t lbn0;		// LSB of number of logical blocks
    uint8_t block_len3;		// MSB of block length in bytes
    uint8_t block_len2;		// MID HIGH of block length in bytes
    uint8_t block_len1;		// MID LOW of block length in bytes
    uint8_t block_len0;		// MSB of block length in bytes
};

#endif // __SCSI_HH__
