/* $Id: s.scsi_none.hh 1.13 03/10/29 16:25:07-05:00 ehallnor@zazzer.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

/* @file
 *  Placeholder for devices that aren't present
 */

#ifndef __SCSI_NONE_HH__
#define __SCSI_NONE_HH__

#include "sim/host.hh"

#include "dev/dma.hh"
#include "dev/scsi_ctrl.hh"
#include "dev/scsi_device.hh"
#include "sim/sim_object.hh"

/*
 * Placeholder for devices that aren't present.  The placeholder is
 * necessary so that a more complete protocol with bus timeouts is not
 * necessary
 */
class ScsiNone : public ScsiDevice
{
  protected:
    ScsiCommand command;		// SCSI command
    DmaPhys phys[SCSI_MAXDMA_LEN];
    DmaRequest dmareq;			// DMA request area
    bool done;				// 0 => no transfer in progress
					// 1 => disk transfer pending
    uint32_t txBytes;			// Return value from operation.
    int error;
    int lun;

    void doCommand();

  protected:
    /*
     * DMA transfer is complete
     */
    class DmaDone : public DmaCallback
    {
      public:
	DmaDone(ScsiNone *d) : device(d) {}
	ScsiNone *device;
	virtual void process() { device->dmaDone(); }
    };

    friend class DmaDone;
    DmaDone dmaDoneCB;

    void dmaDone();

  public:
    ScsiNone(const std::string &name, int tgt);
    ~ScsiNone();

    // OS interface
    bool isDone() const { return true; }
    int bytesTX() const { return 0; }
    int getError() const { return error; }

    virtual int getTarget() const { return target; }

    // Schedule transfers to take place at the correct simulated time
    virtual void startIO(ScsiCommand cmd, DmaPhys *pages, int lun);

    // Tell "controller" that an i/o operation is complete.
    virtual void ioDone();
};


#endif // __SCSI_DISK_HH__
