/* $Id: s.tlaser_clock.hh 1.26 03/12/06 09:57:59-08:00 stever@isabel.reinhardt.house $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

/* @file
 * Interrupt clock generator
 */

#ifndef __TLASER_CLOCK_HH__
#define __TLASER_CLOCK_HH__

#include "sim/eventq.hh"
#include "sim/sim_object.hh"
#include "dev/turbolaser.hh"

class IntrControl;
/*
 * Device model for a hardware interrupt timer
 */
class TlaserClock : public SimObject
{
  protected:
    /*
     * Event signalling the hardware interrupt
     */
    class ClockEvent : public Event
    {
      protected:
	TlaserClock *clock;
	Tick interval;

      public:
	ClockEvent(TlaserClock *clock);

	virtual void process();
	virtual const char *description();
    };

  protected:
    Tick freq;
    Tick delay;
    ClockEvent event;
    bool interrupting[Turbolaser::Max_CPUs];

    IntrControl *intrctrl;

  public:
    TlaserClock(const std::string &name, IntrControl *i, Tick frequency,
		Tick delay);

    void ack(int cpu_id);
    void interrupt();

    bool pending(int cpu_id) { return interrupting[cpu_id]; }
    Tick frequency() const { return freq; }

    virtual void serialize(std::ostream &os);
    virtual void unserialize(Checkpoint *cp, const std::string &section);
};

#endif // __TLASER_CLOCK_HH__




