/* $Id: s.tlaser_node.hh 1.25 04/02/11 16:07:50-05:00 binkertn@zizzer.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

/* @file
 * Turbolaser system bus node (processor, memory, or IO)
 */

#ifndef __TLASER_NODE_HH__
#define __TLASER_NODE_HH__

#include "base/range.hh"
#include "dev/io_device.hh"

class Turbolaser;

/*
 * Turbolaser system bus node (processor, memory, or IO)
 */
class TlaserNode : public PioDevice
{
  public:
    enum Type { KFTHA, KFTIA, MS7CC, SimgleProc4M, SimgleProc16M,
		DualProc4M, DualProc16M };

  private:
    Addr addr;
    static const int size = 0x3000; // from tlaserreg.h

  protected:
    Turbolaser *tlaser;
    int number;
    int tl_cpu_mask;
    Type type;

  public:
    TlaserNode(const std::string &name, Turbolaser *t, int number, Type type,
	       MemoryController *mmu, Addr addr, HierParams *hier, Bus *bus);

    virtual Fault read(MemReqPtr &req, uint8_t *data);
    virtual Fault write(MemReqPtr &req, const uint8_t *data);

    virtual void serialize(std::ostream &os);
    virtual void unserialize(Checkpoint *cp, const std::string &section);

  public:
    Tick cacheAccess(MemReqPtr &req);
};

#endif // __TLASER_NODE_HH__
