/* $Id: s.turbolaser.cc 1.23 03/12/06 09:57:59-08:00 stever@isabel.reinhardt.house $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#include <deque>
#include <string>
#include <vector>

#include "cpu/intr_control.hh"
#include "dev/console.hh"
#include "dev/etherdev.hh"
#include "dev/scsi_ctrl.hh"
#include "dev/tlaser_clock.hh"
#include "dev/tlaser_ipi.hh"
#include "dev/tlaser_mbox.hh"
#include "dev/turbolaser.hh"
#include "sim/builder.hh"
#include "sim/system.hh"

using namespace std;

Turbolaser::Turbolaser(const string &name, ScsiController *s, EtherDev *e,
		       TlaserClock *c, TlaserIpi *i, TlaserMBox *m,
		       SimConsole *con, IntrControl *ic, int intr_freq)
    : SimObject(name), intctrl(ic), cons(con), scsi(s), ethernet(e),
      clock(c), ipi(i), mbox(m), interrupt_frequency(intr_freq)
{
    ipi->setTurbolaser(this);
    for (int i = 0; i < Turbolaser::Max_CPUs; i++)
	intr_sum_type[i] = ipi_pending[i] = 0;
}

void
Turbolaser::serialize(std::ostream &os)
{
    SERIALIZE_ARRAY(intr_sum_type, Turbolaser::Max_CPUs);
    SERIALIZE_ARRAY(ipi_pending, Turbolaser::Max_CPUs);
}
                                                                                
void
Turbolaser::unserialize(Checkpoint *cp, const std::string &section)
{
    UNSERIALIZE_ARRAY(intr_sum_type, Turbolaser::Max_CPUs);
    UNSERIALIZE_ARRAY(ipi_pending, Turbolaser::Max_CPUs);
}

BEGIN_DECLARE_SIM_OBJECT_PARAMS(Turbolaser)

    SimObjectParam<ScsiController *> scsi;
    SimObjectParam<EtherDev *> ethernet;
    SimObjectParam<TlaserClock *> clock;
    SimObjectParam<TlaserIpi *> ipi;
    SimObjectParam<TlaserMBox *> mbox;
    SimObjectParam<SimConsole *> cons;
    SimObjectParam<IntrControl *> intrctrl;
    Param<int> interrupt_frequency;

END_DECLARE_SIM_OBJECT_PARAMS(Turbolaser)

BEGIN_INIT_SIM_OBJECT_PARAMS(Turbolaser)

    INIT_PARAM(scsi, "scsi controller"),
    INIT_PARAM(ethernet, "ethernet controller"),
    INIT_PARAM(clock, "turbolaser clock"),
    INIT_PARAM(ipi, "inter-processor interrupt"),
    INIT_PARAM(mbox, "message box"),
    INIT_PARAM(cons, "system console"),
    INIT_PARAM(intrctrl, "interrupt controller"),
    INIT_PARAM_DFLT(interrupt_frequency, "frequency of interrupts", 1200)

END_INIT_SIM_OBJECT_PARAMS(Turbolaser)


CREATE_SIM_OBJECT(Turbolaser)
{
    return new Turbolaser(getInstanceName(), scsi, ethernet, clock, ipi,
			  mbox, cons, intrctrl, interrupt_frequency);
}

REGISTER_SIM_OBJECT("Turbolaser", Turbolaser)

