/* $Id$ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#include "cpu/exec_context.hh"
#include "cpu/base_cpu.hh"
#include "cpu/full_cpu/bpred.hh"
#include "cpu/full_cpu/full_cpu.hh"
#include "kern/system_events.hh"
#include "sim/system.hh"
#include "sim/sw_context.hh"

void
SkipFuncEvent::process(ExecContext *xc)
{
    Addr newpc = xc->regs.intRegFile[ReturnAddressReg];

    DPRINTF(PCEvent, "skipping %s: pc=%x, newpc=%x\n", description,
	    xc->regs.pc, newpc);

    xc->regs.pc = newpc;
    xc->regs.npc = xc->regs.pc + sizeof(MachInst);

    BranchPred *bp = xc->cpu->getBranchPred();
    if (bp != NULL) {
	bp->popRAS(xc->thread_num);
    }
}


FnEvent::FnEvent(PCEventQueue *q, const std::string & desc, System *system)
    : PCEvent(q, desc), _name(desc)
{
    myBin = system->getBin(desc);
    assert(myBin);
}

void
FnEvent::process(ExecContext *xc) 
{
    if (xc->misspeculating())
        return;
    assert(xc->system->bin && "FnEvent must be in a binned system");
    SWContext *ctx = xc->swCtx;
    DPRINTF(TCPIP, "%s: %s Event!!!\n", xc->system->name(), description);

    if (ctx && !ctx->callStack.empty()) {
	DPRINTF(TCPIP, "already a callstack!\n");
        fnCall *last = ctx->callStack.top();

	if (last->name == "idle_thread")
	    ctx->calls++;

	if (!xc->system->findCaller(myname(), "" ) && 
	    !xc->system->findCaller(myname(), last->name)) {
	    
	    DPRINTF(TCPIP, "but can't find parent %s\n", last->name);
            return;
        }
        ctx->calls--;

	//assert(!ctx->calls && "on a binned fn, calls should == 0 (but can happen in boot)");
    } else {
	DPRINTF(TCPIP, "no callstack yet\n");
        if (!xc->system->findCaller(myname(), "")) {
	    DPRINTF(TCPIP, "not the right function, returning\n");
            return;
        }    
        if (!ctx)  {	    
            DPRINTF(TCPIP, "creating new context for %s\n", myname());
            ctx = new SWContext;
            xc->swCtx = ctx;
        } 
    }
    DPRINTF(TCPIP, "adding fn %s to context\n", myname());
    fnCall *call = new fnCall;  
    call->myBin = myBin;
    call->name = myname();
    ctx->callStack.push(call);
    myBin->activate();
    xc->system->fnCalls++;
    DPRINTF(TCPIP, "fnCalls for %s is %d\n", description, 
	    xc->system->fnCalls.value());
    xc->system->dumpState(xc);
}
