/* $Id$ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#ifndef __TRU64_SYSTEM_HH__
#define __TRU64_SYSTEM_HH__

#include <map>
#include <vector>

#include "sim/system.hh"
#include "targetarch/isa_traits.hh"

class ExecContext;
class EcoffObject;
class SymbolTable;

class BreakPCEvent;
class BadAddrEvent;
class SkipFuncEvent;
class PrintfEvent;
class DebugPrintfEvent;
class DumpMbufEvent;
class FnEvent;
class AlphaArguments;

class Tru64System : public System
{
  private:
    EcoffObject *kernel;
    EcoffObject *console;

    SymbolTable *kernelSymtab;
    SymbolTable *consoleSymtab;

    BreakPCEvent *kernelPanicEvent;
    BreakPCEvent *consolePanicEvent;
    BadAddrEvent *badaddrEvent;
    SkipFuncEvent *skipPowerStateEvent;
    SkipFuncEvent *skipScavengeBootEvent;
    PrintfEvent *printfEvent;
    DebugPrintfEvent *debugPrintfEvent;
    DebugPrintfEvent *debugPrintfrEvent;
    DumpMbufEvent *dumpMbufEvent;

  private:

    Addr kernelStart;
    Addr kernelEnd;
    Addr kernelEntry;
    bool bin;
    std::vector<string> binned_fns;

  public:
    std::vector<RemoteGDB *>   remoteGDB;
    std::vector<GDBListener *> gdbListen;

  public:
    Tru64System(const std::string _name,
                const uint64_t _init_param,
		MemoryController *_memCtrl,
		PhysicalMemory *_physmem,
		const std::string &kernel_path,
		const std::string &console_path,
		const std::string &palcode,
		const std::string &boot_osflags,
		const bool _bin, 
		const std::vector<string> &binned_fns);
    ~Tru64System();

    int registerExecContext(ExecContext *xc);
    void replaceExecContext(ExecContext *xc, int xcIndex);

    Addr getKernelStart() const { return kernelStart; }
    Addr getKernelEnd() const { return kernelEnd; }
    Addr getKernelEntry() const { return kernelEntry; }
    bool breakpoint();

    static void Printf(AlphaArguments args);
    static void DumpMbuf(AlphaArguments args);
};

#endif // __TRU64_SYSTEM_HH__
