/* $Id: s.int_stats.hh 1.19 03/10/15 16:33:22-04:00 binkertn@zans.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#ifndef __INT_STATS_HH__
#define __INT_STATS_HH__

#include <fstream>
#include <string>
#include <vector>
#include <deque>

#include "sim/eventq.hh"
#include "base/range.hh"

class StatsVector
{
  private:
    std::vector<double> stats;

  public:
    StatsVector();
    StatsVector(const StatsVector &sv);
    ~StatsVector();

    bool empty() const { return stats.empty(); }
    void load();
    void dump();

    const StatsVector &operator=(const StatsVector &sv);
    const StatsVector &operator+=(const StatsVector &sv);
    const StatsVector &operator-=(const StatsVector &sv);

    friend StatsVector operator+(const StatsVector &l, const StatsVector &r);
    friend StatsVector operator-(const StatsVector &l, const StatsVector &r);
};

class IntervalStats
{
  private:
    class CycleIntStatEvent : public Event
    {
      private:
	IntervalStats *stats;

      public:
	CycleIntStatEvent(IntervalStats *stats);
	void process();
	virtual const char *description();
    };

    bool enableCycleIntervals;
    bool enableTriggers;

    Range<Counter>    cycleRange;
    std::string         statsFilename;
    std::vector<std::string> statNames;
    Counter           interval;
    bool                exitAfterLastInterval;

    CycleIntStatEvent *event;

    unsigned numThreads;

    unsigned numStats;

    bool valid;

    std::ofstream *	       outfile;
    std::vector<stat_stat_t *> statList;
    std::vector<bool>          dumpValue;
    std::vector<std::string>   statLabels;
    //  we store everything as a double...
    std::vector<double>        lastValue;

    struct stat_sdb_t *        statDB;

    std::string                currentOutputLabel;
    std::deque<std::string>    outputLabelStack;


    //  private methods

    unsigned buildStatList();

  public:
    IntervalStats(stat_sdb_t *sdb, unsigned thread,
		  const std::string &stat_file, bool exit_when_done,
	bool enable_stat_triggers, Counter stat_cycle_int,
        const std::string &stat_range,
        const std::vector<std::string> &statistics);

    void init();

    Counter        getInterval() {return interval;}
    Range<Counter> getRange() {return cycleRange;}
    bool inRange() { return (this && (cycleRange == curTick)); }

    void dumpStats(Tick now);

    void closeOutfile() { delete outfile; valid = false; }
    bool exitWhenDone()
         { return enableCycleIntervals && exitAfterLastInterval; }

    bool isValid() { return valid;}

    void NewInterval(char *c_label) { NewInterval(std::string(c_label)); }
    void NewInterval(std::string label);

    void StartInterval(char *c_label) { StartInterval(std::string(c_label)); }
    void StartInterval(std::string label);
    void EndInterval(char *c_label) { EndInterval(std::string(c_label)); }
    void EndInterval(std::string label);
};

extern IntervalStats *istats;

#endif // __INT_STATS_HH__
