/* $Id: s.system.hh 1.33 04/05/21 13:03:14-04:00 binkertn@ziff.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#ifndef __SYSTEM_HH__
#define __SYSTEM_HH__

#include <string>
#include <vector>

#include "base/loader/symtab.hh"
#include "base/statistics.hh"
#include "cpu/pc_event.hh"
#include "kern/system_events.hh"
#include "sim/sim_object.hh"
#include "sim/sw_context.hh"

class MemoryController;
class PhysicalMemory;
class RemoteGDB;
class GDBListener;

class ExecContext;

class System : public SimObject
{    
    // lisa's binning stuff
  private:
    std::map<const std::string, Stats::MainBin *> fnBins;
    std::map<const Addr, SWContext *> swCtxMap;

  protected:
    std::vector<FnEvent *> fnEvents;

  public:
    Stats::Scalar<> fnCalls;
    Stats::MainBin *Kernel;
    Stats::MainBin *User;

    Stats::MainBin * getBin(const std::string &name);
    bool findCaller(std::string, std::string) const; 

    SWContext *findContext(Addr pcb);
    bool addContext(Addr pcb, SWContext *ctx) {
        return (swCtxMap.insert(make_pair(pcb, ctx))).second;
    }
    void remContext(Addr pcb) { 
        swCtxMap.erase(pcb); 
        return; 
    }
    void dumpState(ExecContext *xc) const;
    
    virtual void serialize(std::ostream &os);
    virtual void unserialize(Checkpoint *cp, const std::string &section);
    

  private:
    std::multimap<const std::string, std::string> callerMap;
    void populateMap(std::string caller, std::string callee);
// 

  public:
    const uint64_t init_param;
    MemoryController *memCtrl;
    PhysicalMemory *physmem;
    bool bin;
    std::vector<string> binned_fns;

    PCEventQueue pcEventQueue;

    std::vector<ExecContext *> execContexts;

    virtual int registerExecContext(ExecContext *xc);
    virtual void replaceExecContext(int xcIndex, ExecContext *xc);

  public:
    System(const std::string _name, const uint64_t _init_param, 
           MemoryController *, PhysicalMemory *, const bool,
	   const std::vector<string> &binned_fns);
    ~System();

    virtual Addr getKernelStart() const = 0;
    virtual Addr getKernelEnd() const = 0;
    virtual Addr getKernelEntry() const = 0;
    virtual bool breakpoint() = 0;

  public:
    ////////////////////////////////////////////
    //
    // STATIC GLOBAL SYSTEM LIST
    //
    ////////////////////////////////////////////

    static std::vector<System *> systemList;
    static int numSystemsRunning;

    static void printSystems();
};

#endif // __SYSTEM_HH__
