/* $Id: s.universe.cc 1.15 04/03/05 05:44:09-05:00 binkertn@ziff.eecs.umich.edu $ */

/*
 * Copyright (c) 2003
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator, developed by Nathan Binkert,
 * Erik Hallnor, Steve Raasch, and Steve Reinhardt, with contributions
 * from Ron Dreslinski, Dave Greene, and Lisa Hsu.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 */

#include <sys/types.h>
#include <sys/stat.h>

#include <cstring>
#include <fstream>
#include <list>
#include <string>
#include <vector>

#include "base/misc.hh"
#include "sim/universe.hh"
#include "sim/host.hh"
#include "sim/param.hh"

using namespace std;

Tick curTick = 0;
Tick ticksPerSecond;
double __ticksPerMS;
double __ticksPerUS;
double __ticksPerNS;
double __ticksPerPS;

string outputDirectory;
ostream *outputStream;

class UniverseParamContext : public ParamContext
{
  private:
    ofstream outputFile;

  public:
    UniverseParamContext(const string &is) : ParamContext(is) {}
    void checkParams();
};

UniverseParamContext universe("Universe");

Param<Tick> universe_freq(&universe, "frequency", "tick frequency",
			  200000000);

Param<string> universe_output_dir(&universe, "output_dir",
				  "directory to output data to");
Param<string> universe_output_file(&universe, "output_file",
				   "file to dump simulator output to");

void
UniverseParamContext::checkParams()
{
    ticksPerSecond = universe_freq;
    double freq = double(ticksPerSecond);
    __ticksPerMS = freq / 1.0e3;
    __ticksPerUS = freq / 1.0e6;
    __ticksPerNS = freq / 1.0e9;
    __ticksPerPS = freq / 1.0e12;

    if (universe_output_dir.isValid()) {
	outputDirectory = universe_output_dir;

	// guarantee that directory ends with a '/'
	if (outputDirectory[outputDirectory.size() - 1] != '/')
	    outputDirectory += "/";

	if (mkdir(outputDirectory.c_str(), 0777) < 0) {
	    if (errno != EEXIST) {
		panic("%s\ncould not make output directory: %s\n",
		      strerror(errno), outputDirectory);
	    }
	}
    }

    string filename;
    if (universe_output_file.isValid()) {
	string f = universe_output_file;
	if (f != "stdout" && f != "cout" && f != "stderr" && f != "cerr")
	    filename = outputDirectory + f;
	else
	    filename = f;
    } else {
	if (outputDirectory.empty())
	    filename = "stdout";
	else
	    filename = outputDirectory + "output.txt";
    }

    if (filename == "stdout" || filename == "cout")
	outputStream = &cout;
    else if (filename == "stderr" || filename == "cerr")
	outputStream = &cerr;
    else {
	outputFile.open(filename.c_str(), ios::trunc);
	outputStream = &outputFile;
    }
}
