#!/usr/bin/perl

require "/home/sraasch/poolfs/abrevs.pl";

$max_name_length = 10;
$first = 1;

use Getopt::Std;

getopts('DAPxtc:f:');

$debug = 0;
if ($opt_D) {
    $debug = 1;
}

$fetch_width = 8;
if ($opt_f) {
    $fetch_width = $opt_f;
}

#
# Define the mapping from causes generated by the simulator to the categories.
#
%cause_categories =
(
 # iq wierdness
 'lsq_full{queue}'           => 'other',
 'iq_full{queue}'            => 'other',

 'iq_full{issuebw}'       => 'I:BW',
 'lsq_full{issuebw}'      => 'I:BW',

 'iq_full_deps{rd_port}'  => 'QD:M',
 'lsq_full_deps{rd_port}' => 'QD:M',

 'rob_full{commit_bw}'      => 'C:BW',
 'rob_full{storebuf}'      => 'Q:SB',

 'qfull{ireg_full}'             => 'Regs',
 'qfull{fpreg_full}'            => 'Regs',
 'qfull{no_inst}'               => 'other',
 'qfull{rob_cap}'               => 'Polcy',
 'qfull{iq_cap}'                => 'Polcy',
 'qfull{bw}'                    => 'D:BW',
 'qfull{policy}'                => 'D:Pol',

 'fetch{brlim}'                 => 'BrLim',
 'fetch{invpc}'                 => 'MisSp',
 'fetch{btb_miss}'              => 'MisSp',
 'fetch{brrecover}'             => 'MisSp',
 'fetch{sync}'                  => 'Sync',
 'fetch{lowconf}'               => 'LoCnf',
 'fetch{policy}'                => 'F:Pol',
 'fetch{none}'                  => 'none',
 'fetch{bandwidth}'             => 'F:BW',
 'fetch{unknown}'               => 'Broken',
 'fetch{zero_prio}'             => 'F:Pol',

 #
 # something is probably Broken is these are non-zero
 #
 'iq_full_deps{none}'     => 'Broke',
 'lsq_full_deps{none}'    => 'Broke',
 'iq_full_deps{wr_port}'  => 'Broke',
 'lsq_full_deps{wr_port}' => 'Broke',
 'iq_full_fu{none}'       => 'Broke',
 'lsq_full_fu{none}'      => 'Broke',
 'rob_full_fu{none}'      => 'Broke',
 'rob_full{rob_empty}'    => 'Broke',

);

#
# Define the mapping from causes generated by the simulator to the categories.
#
%translate =
(
 # iq wierdness
 'qfull_lsq_full_iq'           => 'lsq_full{queue}',
 'qfull_iq_full_iq'            => 'iq_full{queue}',

 'qfull_iq_full_issuebw'       => 'iq_full{issuebw}',
 'qfull_lsq_full_issuebw'      => 'lsq_full{issuebw}',

 'qfull_iq_full_deps_rd_port'  => 'iq_full_deps{rd_port}',
 'qfull_lsq_full_deps_rd_port' => 'lsq_full_deps{rd_port}',

 'qfull_rob_full_commitbw'     => 'rob_full{commit_bw}',
 'qfull_rob_full_storebuf'     => 'rob_full{storebuf}',

 'qfull_ireg_full'             => 'qfull{ireg_full}',
 'qfull_fpreg_full'            => 'qfull{fpreg_full}',
 'qfull_no_inst'               => 'qfull{no_inst}',
 'qfull_rob_cap'               => 'qfull{rob_cap}',
 'qfull_iq_cap'                => 'qfull{iq_cap}',
 'qfull_bw'                    => 'qfull{bw}',
 'qfull_policy'                => 'qfull{policy}',

 'brlim'                       => 'fetch{brlim}',
 'invpc'                       => 'fetch{invpc}',
 'btb_miss'                    => 'fetch{btb_miss}',
 'brrecover'                   => 'fetch{brrecover}',
 'sync'                        => 'fetch{sync}',
 'lowconf'                     => 'fetch{lowconf}',
 'policy'                      => 'fetch{policy}',
 'none'                        => 'fetch{none}',
 'bandwidth'                   => 'fetch{bandwidth}',
 'unknown'                     => 'fetch{unknown}',
 'zero_prio'                   => 'fetch{zero_prio}',

 'qfull_iq_full_deps_none'     => 'iq_full_deps{none}',
 'qfull_lsq_full_deps_none'    => 'lsq_full_deps{none}',
 'qfull_iq_full_deps_wr_port'  => 'iq_full_deps{wr_port}',
 'qfull_lsq_full_deps_wr_port' => 'lsq_full_deps{wr_port}',
 'qfull_iq_full_fu_none'       => 'iq_full_fu{none}',
 'qfull_lsq_full_fu_none'      => 'lsq_full_fu{none}',
 'qfull_rob_full_fu_none'      => 'rob_full_fu{none}',
 'qfull_rob_full_rob_empty'    => 'rob_full{rob_empty}',

);

if ($#ARGV < 0) {
    print "Need at least one argument...\n";
    usage();
    exit;
}

$cpuname = "cpu";
if ($opt_c) {
    $cpuname = $opt_c;
}


# default categories to print
@categories = qw(IPC MisSp D:BW IQ I:BW QD:FU QD:M Q:Mem Q:FU Q:SB D:Pol ICach Broke);



sub mergeCauses
{
    my $cause = shift;
    my $cat;

    #  strip off the unused prefix...
    my $match = $cpuname."_floss_\\d+_";
    $cause =~ s/$match//;

    print "Merging Cause \'$cause\'..." if $debug;

    #
    # look up in %cause_categories hash first...
    # (if nothing there, then we check the matching below)
    #
    if ($cat = $cause_categories{$cause}) {
	print "   found cause category in hash\n" if $debug;
	return $cat;
    }

    #
    #  A couple more Broken values
    #
    elsif ($cause =~ /Broken/i) {
	return 'Broke';
    }
    elsif ($cause =~ /cache_hit/) {
	return 'Broke';
    }
    elsif ($cause =~ /dcache_not_issued/) {
	return 'Broke';
    }
    elsif ($cause =~ /(iq|lsq)_full{noinsts}/) {
       return 'IQ';
    }


    #
    #  Easy to blame stuff on the I-Cache
    #
    elsif ($cause =~ /icache/) {
	return 'ICach';
    }

    #
    #  If we're out of read- or write-ports
    #
    elsif ($cause =~ /(iq|lsq|rob)_full_fu{(rd|wr)_port}/) {
	return 'Q:Mem';
    }

    #
    #  Any other FU problems
    #
    elsif ($cause =~ /(iq|lsq|rob)_full_fu/) {
	return 'Q:FU';
    }

    #
    #  We've already filtered out deps on memory ops...
    #
    elsif ($cause =~ /(iq|lsq)_full_deps/) {
	return 'QD:FU';
    }

    #
    #  Queue filled due to D-Cache problem
    #
    elsif ($cause =~ /(iq|lsq|rob)_full_dcache/) {
	return 'Q:Mem';
    }

    #
    #  An instruction has been in the queue long enough to issue
    #
    elsif ($cause =~ /(iq|lsq)_full{tooyoung}/) {
	return 'Q:Age';
    }

    #
    #  An instruction wasn't issued due to inorder restrictions
    #
    elsif ($cause =~ /(iq|lsq)_full{issueinorder}/) {
	return 'InOrd';
    }

    #
    #  If we can't figure out how to categorize this...
    #
    else {
	die "No category for $cause";
    }
}


# ====================================================================


while ($fn = shift) {
    @names = ();

    #
    #  Read the stats file and get the number of cycles
    #
    read_stats($fn, "stats");

    $num_cycles = ${"stats::sim_ticks"};

    print "simulated $num_cycles cycles\n" if $debug;

    #
    # Figure out how many threads we're dealing with...
    #
    @k = keys %stats:: ;
    my $match = $cpuname."_fetch_branch_count";
    @temp = grep /$match/, @k;

    $num_threads = $#temp + 1;
    if ($num_threads > 1) {
	#  correct for the total entry...
	$num_threads = $num_threads - 1; 
    }

    print "found $num_threads threads in file \'$fn\'\n" if $debug;

    #
    #  Now, try to get the names of the threads that are in the file
    #
    #  NOTE:  This only works for runs generated from the "standard" EIO files!!!
    #
    open IN, "<$fn" or die "Can't open \'$fn\'";

    {
	my $debug = 0;
	my @pieces = split /\./, $fn;
	
	print "$fn -> pieces: @pieces\n" if $debug;
	# point to the first bm abbreviation in filename
	my $idx = $#pieces - 2;
	my $front_idx = $#pieces - 2 - $num_threads;

	@f = split /\//, $pieces[$front_idx];
	$front = $f[$#f];

	print "  starting idx is $idx\n" if $debug;
	for (my $i=$num_threads-1; $i>=0; $i--) {
	    print "    assigning $pieces[$idx] to names[$i]\n" if $debug;
	    $names[$i] = $front . "." . $fullname{ $pieces[$idx] };
	    $idx--;
	}
	print "  done.\n" if $debug;
    }

    $max_name_length = 11 + length($front);

    print "Found $num_threads threads\n" if $debug;

    for (my $t=0; $t<$num_threads; ++$t) {

	if (! $names[$t]) {
	    $names[$t] = "T$t";
	}

	print "Examining thread $t ($names[$t]):\n" if $debug;

	#
	#  Add in the actual IPC
	#
	if ($num_threads > 1) {
	    $floss[$t]{IPC} = ${"stats::$cpuname"."_com_ipc[$t]"};
	    $fetch_rate = ${"stats::$cpuname"."_fetch_rate[$t]"};
	}
	else {
	    $floss[$t]{IPC} = ${"stats::$cpuname"."_com_ipc"};
	    $fetch_rate = ${"stats::$cpuname"."_fetch_rate"};
	}
	$floss[$t]{MisSp} = $fetch_rate - $floss[$t]{IPC};

	$real_total[$t] = $floss[$t]{IPC} + $floss[$t]{MisSp};

	#
	#  now get the FLOSS info...
	#
	$old_format = 0;

	$match =  $cpuname."_floss_".$t."_";

	my @causes = grep /$match/, @k;
	if ($#causes == -1) {
	    print "seems to be OLD FORMAT!\n" if $debug;
	    $old_format = 1;

	    $match =  "fetch_loss_counters\\[".$t."\\]";
	    @causes = grep /$match/, @k;
	}

	print "Thread $t got $#causes causes\n" if $debug;

	my $temp = 0;
	foreach $c (@causes) {
	    my $val = ${"stats::".$c};
	    print "Cause \'$c\' = $val\n" if $debug;

	    if ($val > 0) {
		my $lookup = $c;
		my $v = $val / $num_cycles;
		
		$temp += $v;
		
		printf "  (%6.2f)  ", $v if $debug;

		my $blame;
		if ($old_format) {
		    $blame = old_mergeCauses($lookup);
		}
		else {
		    $blame = mergeCauses($lookup);
		}
		print "\'$blame\'\n" if $debug;

		#
		# Add this cause to the results
		#
		$floss[$t]{$blame} += $v;
		$real_total[$t] += $v;

		$all_causes{$blame} = 1;
	    }
	    else {
		print "zero\n" if $debug>1;
	    }
	}
	print "total = $temp\n" if $debug;
    }


    #
    #  Munge category list to print out the info we want...
    #
    if ($opt_A) {
	@categories = ();
	foreach $k (keys %all_causes) {
	    next if ($k =~ /MisSp/);
	    push @categories, $k;
	}
	sort @categories;

	# we always want these two first...
	@categories = ("IPC", "MisSp", @categories);
    }


    #
    #  Print out the values we collected above based on the @categories list,
    #  the $opt_x (Excel output), and $opt_t (print totals)
    #
    if ($opt_x) {
	print_csv();
    }
    else {
	print_human($first);
	$first = 0;
    }

    undef @floss;
}

exit;


sub print_human {
    my ($print_hdr) = @_;

    my $format = "%".$max_name_length."s";
    my $grand_total = 0;
    my $grand_real_total = 0;
    my %col_total;

    if ($print_hdr) {
	printf $format, "";
	print "  ";
	foreach $col (@categories) {
	    printf "%6s ", $col;
	}
	print " Total" if $opt_t;
	print "\n";
    }

    for ($t=0; $t<$num_threads; ++$t) {
	$row_total = 0;

	printf $format.": ",$names[$t];
	foreach $col (@categories) {
	    $val = $floss[$t]{$col};
	    $row_total += $val;
	    $col_total{$col} += $val;

	    printf "%6.2f ", $val;
	}
	printf " %6.2f (%6.2f)", $row_total, $real_total[$t] if $opt_t;

	if ($row_total < (0.95 * $real_total[$t])) {
	    print " ** LOW ** ";
	}
	print "\n";

	$grand_total += $row_total;
	$grand_real_total += $real_total[$t];
    }

    if ($opt_t and ($num_threads > 1)) {
	printf $format, "Total: ";
	foreach $col (@categories) {
	    printf "%6.2f ", $col_total{$col};
	}
	printf "%6.2f (%6.2f)\n", $grand_total, $grand_real_total;
    }

    if ($grand_total < (0.95 * $fetch_width)) {
	print "** LOW **\n";
    }
}


sub print_csv {
    my $grand_real_total = 0;

    print "Thread,";
    foreach $col (@categories) {
	print "$col,";
    }
    print "Total" if $opt_t;
    print "\n";
    for ($t=0; $t<$num_threads; ++$t) {
	$row_total = 0;
	print "$names[$t],";
	foreach $col (@categories) {
	    $val = $floss[$t]{$col};
	    $row_total += $val;
	    $col_total{$col} += $val;

	    printf "%.2f,", $val;
	}
	printf "%.2f,", $row_total if $opt_t;
	print "LOW" if ($row_total < (0.95 * $real_total[$t]));
	print "\n";

	$grand_total += $row_total;
	$grand_real_total += $real_total[$t];
    }

    if ($opt_t and ($num_threads > 1)) {
	print "Total,";
	foreach $col (@categories) {
	    printf "%.2f,", $col_total{$col};
	}
	printf "%.2f,", $grand_total;
    }
    print "LOW" if ($grand_total < (0.95 * $grand_real_total));
    print "\n";
}

#==============================================================================

#
# Read in the stats file.  Regular scalar statistics are read into the variable
# of the same name (e.g., $sim_cycle).  Scalar stats that end in _<digit> are
# assumed to be per-thread and are read into an array 
# (e.g., IPC_0 --> $IPC[0]).
#
# If an argument is supplied, it is assumed to be a perl regexp. Only stats that
# match the regexp are sucked in
#
# Distributions are read into hashes; if the name of the distribution ends in
# _<digit> an array of hashes is used.  The fetch_loss_counters dist is handled
# specially (since it's not a standard SimpleScalar distribution).
#
# Mapping from stat name to variable name:
#  - Periods, dashes, and colons are replaced with underscores
#  - per-thread stats (foo_0) are changed to foo[0]
#  - Names forced to lower-case
#
sub read_stats
{
    my $debug = 0;
    if ($opt_P) {
	print "\ndebugging parse\n";
	$debug = 1;
    }


    #
    #  Get params
    #
    my $file = shift;
    my $pkg = shift;
    my ($regexp) = @_;


    my $bin_number = 0;
    my $bin_hdr = "";

    #
    #  Constants (for matching)
    #
    my $CMD_LINE     = '^command line:';
    my $STATS_HEADER = '- Begin Simulation Statistics -';
    my $BINNING_FLAG = 'PRINTING BINNED STATS';
    my $BINNING_HEADER = '---(.+?)\sBin---';   # minimal match
    my $BINNING_FOOTER = '---------------------';
    my $FOOTER       = '--  End Simulation Statistics  --';

    my $binning = 0;
    my $bin_number;

    #
    #  Clear destination for stats
    #

    if ($pkg ne "stats") {
	die "you'd better find a way to make the clearing-out stuff work...";
	#  see the section immediately below this!
    }

    #
    #  CLEARING OUT STUFF
    #
    undef %stats::;
    foreach $sym (keys %stats::) {
	print "found \$stats::{$sym} (value is $stats::{$sym})\n";
    }

    print "Creating variables in package $pkg from file $file\n" if $debug;

    open(FILE, "<$file") or die "Can't open $file: $!";

    my $phase = 0;
    while (<FILE>) {

	if ($phase == 0) {
	    if (/$STATS_HEADER/) {
		$phase = 1;
	    }
	    next;
	}

        #
        #  "normal" statistical value
        #
	my $stat;
	my $dist;
	my $val;
	my $subscript;
	
	if (($stat, $val) = /^([\w\.\:]+)\s+(\d+\.?\d*\%*)/) {

            # Replace non-word chars with underscores
            $stat =~ s/\W/_/g;

            # If this is a per-thread stat (like blah_1)
            # change to blah[1]
            if ($stat =~ /_(\d)$/) {
                $subscript = $1;
                $stat =~ s/_\d$//;
            } else {
                undef $subscript;
            }

            #  replace any remaining dashes with underscore
            $stat =~ s/\-/_/g;

	    # force stat name to lower-case
	    $stat = lc $stat;

	    #
	    #  Magic happens here... create the stat name in the specified package
	    #                        and give it the value we grabbed earlier
	    #
            if (defined $subscript) {
		$statname = $stat."[$subscript]";
            } else {
		$statname = $stat;
            }

	    print "$statname = $val" if $debug;

	    $doit = 1;
	    if (($regexp ne "") and ($statname !~ /$regexp/)) {
		$doit = 0;
	    }

	    if ($doit) {
		$ { $pkg . "::" . $bin_hdr . $statname } = $val;
	    }
	    else {
		print " ... doesn't match regexp \'$regexp\'" if $debug;
	    }
	    print "\n" if $debug;

	}

        #
        #  If we are starting to read a distribution...
        #
	elsif (($dist) = /^([\w\.\:]+).start/) {

            # Replace periods with underscores
            $dist =~ s/\./_/g;
	    $dist =~ s/:/_/g;

	    $dist = lc $dist;

            #  Handle per-thread distributions as an array of hashes
            if ($dist =~ /_(\d)$/) {
                $subscript = $1;
                $dist =~ s/_\d$//;
            } else {
                undef $subscript;
            }

	    while ($line = <FILE>) {
		chomp $line;
		last if $line =~/\.end/;

                print "DIST: $line\n" if $debug;

		# spaces in the name are bad... hope there isn't more than one
		# ... or less than one between name and value
		$line =~ s/(\w) (\w)/$1_$2/g;
		$line =~ s/^\s+//;

		my $label, $value;
		($label, $value) = split /\s+/, $line;
		$label = lc $label;

                # replace dash with underscore
                $label =~ s/\-/_/g;
		$label =~ s/:/_/g;

                if (defined $subscript) {
                    print "  -> $bin_hdr"."$dist\[$subscript]{$label} = $value\n" 
		      if $debug;

                    $ { $pkg . "::" . $bin_hdr . $dist."[$subscript]{$label}" } = $value;
                } else {
                    print "  -> $bin_hdr"."$dist\{$label} = $value\n" if $debug;
                    $ { $pkg . "::" . $bin_hdr . $dist."{$label}" } = $value;
                }
	    }
        } 
    }

    close(FILE);

    print "\n\nDone reading stat-file\n\n" if $debug;

    if (($bin_number > 0) and $opt_b) {
	print "Bin identification:\n";
	for($i=0; $i<$bin_number; ++$i) {
	    print "  bin $i = $bin_name[$i]\n";
	}
    }

    $num_bins = $bin_number;
}


sub usage {
    print <<EOF;

   bkdwn.pl [options] <cpu_name> <stats_file> [<stats_file> ...]

      Options:
         -x   = output Excel-format data (CSV)
         -t   = print row and column totals
         -c <string> = specify a different CPU name
         -A   = print all causes
         -D   = debug the main program
         -P   = debug the stats grabber

EOF
}

# ====================================================================
#
#   OLD compatibility
#
#



sub old_mergeCauses
{
    my $cause = shift;
    my $cat;

    print "OLD: Merging Cause \'$cause\'..." if $debug;

    #
    #  Translate the old-style naming....
    #
    $cause =~ s/fetch_loss_counters\[\d+\]\{//;
    $cause =~ s/\}$//;
    if ($translate{$cause}) {
	$cause = $translate{$cause};
    }


    #
    # look up in %cause_categories hash first...
    # (if nothing there, then we check the matching below)
    #
    print " (looking up $cause) " if $debug;
    if ($cat = $cause_categories{$cause}) {
	print "   found cause category in hash\n" if $debug;
	return $cat;
    }

    #
    #  A couple more Broken values
    #
    elsif ($cause =~ /Broken/i) {
	return 'Broke';
    }
    elsif ($cause =~ /cache_hit/) {
	return 'Broke';
    }
    elsif ($cause =~ /dcache_not_issued/) {
	return 'Broke';
    }
    elsif ($cause =~ /(iq|lsq)_full_noinsts/) {
       return 'IQ';
    }


    #
    #  Easy to blame stuff on the I-Cache
    #
    elsif ($cause =~ /icache/) {
	return 'ICach';
    }

    #
    #  If we're out of read- or write-ports
    #
    elsif ($cause =~ /(iq|lsq|rob)_full_fu_(rd|wr)_port/) {
	return 'Q:Mem';
    }

    #
    #  Any other FU problems
    #
    elsif ($cause =~ /(iq|lsq|rob)_full_fu/) {
	return 'Q:FU';
    }

    #
    #  We've already filtered out deps on memory ops...
    #
    elsif ($cause =~ /(iq|lsq)_full_deps/) {
	return 'QD:FU';
    }

    #
    #  Queue filled due to D-Cache problem
    #
    elsif ($cause =~ /(iq|lsq|rob)_full_dcache/) {
	return 'Q:Mem';
    }

    #
    #  An instruction has been in the queue long enough to issue
    #
    elsif ($cause =~ /(iq|lsq)_full_tooyoung/) {
	return 'Q:Age';
    }

    #
    #  An instruction wasn't issued due to inorder restrictions
    #
    elsif ($cause =~ /(iq|lsq)_full_issueinorder/) {
	return 'InOrd';
    }

    #
    #  If we can't figure out how to categorize this...
    #
    else {
	die "No category for $cause";
    }
}


