#!/usr/bin/perl

use Getopt::Std;

require "$ENV{HOME}/abrevs.pl";


$debug = 0;


getopts('Ddxoqibr:');

@files = @ARGV;

if ($#ARGV < 0) {
  print "Usage:   $0 [-iqb][-r <filename>] <filename> [<filename> ... ] \n";
  print "              Calculates the individual thread EPR values\n";
  print "              and the total workload EPR value.\n";
  print "          -r  Reference different baseline IPC file\n";
  print "          -q: Quiet... Don't print file-names\n";
  print "          -o: One-line output\n";
  print "          -i: Print IPC values also\n";
  print "          -x: Excel output\n";
  print "          -d: print divider prior to *.1.out file results\n";
  print "          -D: print debugging output\n";
  print "          -b: Summarize values by benchmark\n\n";
  exit;
}

if ($opt_D) {
    $debug = 1;
}

if ($opt_b) {
    print "gotta fix this!";
    exit;
}

if ($opt_x) {
    print "# Filename, per-thread EPR, total EPR";
    if ($opt_i) {
	print ", per-thread IPC, total IPC";
    }
    print "\n";
}

foreach $file ( @files ) {
    undef @eprs, @ipcs;

    $num_files++;

    if ($opt_d && ($num_files > 1) && ($file =~ /.1.out$/)) {
	print "...............................................................................................................\n";
    }

    @bms  = get_epr_values($file, "NAME", $opt_r);
    @eprs = get_epr_values($file, "EPR", $opt_r);
    if ($opt_i) {
        @ipcs = get_epr_values($file, "IPC", $opt_r);
    }

    $num_threads = scalar(@bms);

    if ($opt_x) {
	print "$file, ";

	for ($i=0; $i<$num_threads; $i++) {
	    printf(" %.4f,",$eprs[$i]);
	}
	printf " %.4f,", $eprs[$num_threads];

	if ($opt_i) {
	    for ($i=0; $i<$num_threads; $i++) {
		printf(" %.4f,",$ipcs[$i]);
	    }
	    printf "  %.4f", $ipcs[$num_threads]; 
	}

	print "\n";
    }
    else {
	if (not $opt_q) {
	    print "File: $file\n";
	}

	for ($i=0; $i<$num_threads; $i++) {
	    printf("%s : %.4f (EPR)",$bms[$i],$eprs[$i]);
	    if ($opt_i) {
		printf(", %.4f (IPC)",$ipcs[$i]);
	    }
	    print "\t";
	    print "\n" if not $opt_o;
	}
	printf "Sum: %.4f (EPR)", $eprs[$num_threads];
	if ($opt_i) {
	    printf ", %.4f (IPC)", $ipcs[$num_threads]; 
	}
	print "\n";
	print "\n" if not $opt_o;
    }
}



#===========================================================================

my %base_ipc = {};
my %base_msipc = {};

sub get_epr_values {

#    my $debug = 0;

    my ($file, $mode, $baseline_file) = @_;
    my $line, $num_threads=0, @thread_name;
    my $stat;

    my @rval;
    my $epr_total = 0;
    my $ipc_total = 0;


    if ($debug) {
	print "get_epr_values() called with file=$file, mode=$mode, baseline_file=$baseline_file\n";
    }

    #
    #  Look for "epr.conf" in current directory. 
    #  Use the file specified there for baseline
    #  data...
    #
    if (-e "epr.conf") {
	open C, "<epr.conf" or die "Can't open epr.conf";
	$baseline_file = <C>;
	close C;
	chomp $baseline_file;
    }

    if ($baseline_file eq "") {
	$baseline_file = "$ENV{HOME}/dtw/results/baseline.ipc";
    }


    #
    #  Figure out how many threads in this file...
    #  Along the way, grab the benchark name...
    #
    $num_threads = 0;
    open IN, "<$file" or die "Can't open $file for reading!";
    while ( $line = <IN> ) {
        chomp $line;

        if ($line =~ /^workload=(.+)$/) {
	    @a = split / /, $1;
	    foreach $b (@a) {
		$c = $b;
		$c =~ s/cpu.//;

		# the workload name may have one or more digits appended..
		# --> remove them...
		$c =~ s/\d*$//;

		print "Looking up abbreviation for \"$c\"...\n" if $debug;
		$thread_name[$num_threads] = $abrev{$c};

		print STDERR "found $b ($thread_name[$num_threads]) as thread #$num_threads\n" if $debug;
		$num_threads++;
	    }
	    last;
        }
        last if ($line =~ /simulation started/);
    }
    close IN;

    if ($mode eq "NAME") {
        return @thread_name;
        exit;
    }

    print "Looking for $num_threads threads...\n" if $debug;

    #
    #  Just do this once...
    #
    @k = keys %base_ipc;
    if ($base_ipc{$k[0]} == 0) {
	print "Getting base IPC data... \n" if $debug;
        get_base_ipc_data(\%base_ipc, \%base_msipc, $baseline_file);
    }

    #
    #  Get the individual IPC's for the threads
    #
    for ($i=0; $i<$num_threads; $i++) {
        $stat = `~sraasch/Public/bin/get -q cpu.COM:IPC_$i -- $file`;
        chomp $stat;
        print STDERR "get returned '$stat' for IPC\n" if $debug;

        if ($stat =~ /(\d+\.\d+)\s+$/) {
            # Got a value
            
            $ipc = $1;

            print STDERR "IPC($thread_name[$i]) = $ipc\n" if $debug;

            if ($mode eq "EPR") {
                if ( $base_ipc{$thread_name[$i]} ) {
                    $epr = $ipc / $base_ipc{$thread_name[$i]};
                    $epr_total += $epr;
                    @rval = (@rval, $epr);
                }
                else {
                    @rval = (@rval, "N/A");
                }
            }
            elsif ($mode eq "IPC") {
                $ipc_total += $ipc;
                @rval = (@rval, $ipc);
            }
        }
        else {
              @rval = (@rval, "N/A");
        }
    }

    if ($mode eq "EPR") {
        @rval = (@rval, $epr_total);
    }
    elsif ($mode eq "IPC") {
        @rval = (@rval, $ipc_total);
    }

    print "Returning @rval\n" if $debug;

    return @rval;
}


#
#  Grab the base benchmark IPC's from the default location
#
sub get_base_ipc_data {

    my $debug = 0;

    my($ipc_hash_ref, $msipc_hash_ref, $baseline_file) = @_;

    open IN, "<$baseline_file" or
      die "Cannot open base IPC's for reading ($baseline_file) ";

    while ( $line = <IN> ) {
        chomp $line;

        next if $line =~ /^#/;
	next if not $line =~ /\S/;

        if ($line =~ /\.(\w+)\.\d+\.out\s*:\s+(\d+\.\d+)\s+(\d+\.\d+)/) {
            print STDERR "BASE: IPC{$1} = $2  MSIPC{$1} = $3\n" if $debug;
            $$ipc_hash_ref{$1} = $2;
            $$msipc_hash_ref{$1} = $3;
        }
	elsif ($line =~ /huh/) {
            print STDERR "BASE: undefined values\n" if $debug;
            $$ipc_hash_ref{$1} = 1;
            $$msipc_hash_ref{$1} = 1;
	}
        else {
            close IN;
            print STDERR "\"$line\"\n";
            print "Error: syntax in base IPC file: \"$line\"\n";
	    exit;
        }
    }
    close IN;
}
