#!/usr/bin/env python
import numarray as nm
import MySQLdb, getpass, re, sys, types

cursor = None
def dbConnect():
    # connect
    db = MySQLdb.connect(db=dbConnect.db,
                         host=dbConnect.host,
                         user=dbConnect.user,
                         passwd=dbConnect.passwd)

    # create a cursor
    global cursor
    cursor = db.cursor()

dbConnect.host = 'localhost'
dbConnect.user = getpass.getuser()
dbConnect.passwd = ''
dbConnect.db = 'm5stats'

# Name: listbins
# Desc: Prints all bins matching regex argument, if no argument
#       is given all bins are returned
def listBins(regex='.*'):
    print "Bins"
    print "----------------------------------------"

    cursor.execute("select bn_name from bins")
    res = cursor.fetchall()
    rx = re.compile(regex)
    for r in res:
        if rx.match(r[0]):
             print r[0]


# Name: listruns
# Desc: Prints all runs matching a given user, if no argument
#       is given all runs are returned
def listRuns(user=None):
    print "Runs"
    print "----------------------------------------"
    if user is None:
        sql = "select rn_name, rn_user, rn_id from runs order by rn_name"
    else:
        sql = 'select rn_name, rn_user, rn_id from runs where rn_user = "' \
              + user + '" order by rn_name'
    cursor.execute(sql)
    res = cursor.fetchall()
    for r in res:
         print '%-25s %-10s %-5d' % (r[0], r[1], r[2])

# Name: listsamples
# Desc: Prints all samples for a given run

def listSamples(run):
    print "Samples"
    print "----------------------------------------"
    sql = 'select distinct dt_sample from data where dt_run = ' + str(run)
    cursor.execute(sql)
    res = cursor.fetchall()
    for r in res:
         print r[0]

# Name: liststats
# Desc: Prints all statistics that appear in the database,
#         the optional argument is a regular expression that can
#         be used to prune the result set
def listStats(regex='.*'):
    print "Stats"
    print "----------------------------------------"

    cursor.execute("select st_name from stats")
    res = cursor.fetchall()
    rx = re.compile(regex)
    for r in res:
        if rx.match(r[0]):
             print r[0]

# Name: getStatFromName
# Desc: given a regular expression for matching statistics,
#       this function returns an array of stat ids of matching
#        statistics
def getStatFromName(statname):
    sql = 'select st_id, st_name from stats order by st_name'
    cursor.execute(sql)
    res = cursor.fetchall()
    rx = re.compile(statname)
    stats = []
    for r in res:
        if rx.match(r[1]):
           stats.append(r[0])

    return stats


# Name: getRunFromName
# Desc: given a regular expression for matching runs,
#       this function returns an array of run ids of matching
#        statistics
def getRunFromName(runname):
    sql = 'select rn_id, rn_name from runs order by rn_name'
    cursor.execute(sql)
    res = cursor.fetchall()
    rx = re.compile(runname)
    runs = []
    for r in res:
        if rx.match(r[1]):
           runs.append(r[0])

    return runs


# Name: getBinsFromName
# Desc: given a regular expression for matching bins,
#       this function returns an array of bin ids of matching
#        statistics
def getBinsFromName(binname):
    sql = 'select bn_id,bn_name from bins order by bn_name'
    cursor.execute(sql)
    res = cursor.fetchall()
    rx = re.compile(binname)
    bins = []
    for r in res:
        if rx.match(r[1]):
            bins.append(r[0])

    return bins

# Name: getNotBinsFromName
# Desc: given a regular expression for matching bins,
#       this function returns an array of bin ids that don't match
def getNotBinsFromName(binname):
    sql = 'select bn_id,bn_name from bins order by bn_name'
    cursor.execute(sql)
    res = cursor.fetchall()
    rx = re.compile(binname)
    bins = []
    for r in res:
        if not rx.match(r[1]):
            bins.append(r[0])

    return bins
# Name: getAllRuns
# Desc: Returns an array of all runs in the database
def getAllRuns():
    cursor.execute('select rn_id from runs')
    runs = []
    res = cursor.fetchall()
    for r in res:
        runs.append(r[0])
    return runs

# Name: getAllBins
# Desc: Returns an array of all bins in the database
def getAllBins():
    cursor.execute('select bn_id from bins')
    bins = []
    res = cursor.fetchall()
    for r in res:
        bins.append(r[0])
    return bins

# Name: getAllSamples
# Desc: returns a list of all samples for a given run
def getAllSamples(run):
    cursor.execute('select distinct dt_sample from data where dt_run = ' + str(run))
    samples = []
    res = cursor.fetchall()
    for r in res:
        samples.append(r[0])
    return samples

# Name: sumAvg
# Desc: given a run, a stat and an array of samples and bins,
#        sum all the bins and then average the samples for non-binned
#        runs this will just return the average of samples, however
#        a bin array still must be passed
def sumAvg(run,stat,samples=None,bins=None):
    sql = '''\
    select dt_sample as sample, sum(dt_data) as data
    from data
    where dt_run=%d''' % run

    if isinstance(stat, types.ListType):
        val = ' or '.join(map(lambda x: 'dt_stat=' + str(x), stat))
        sql += ' and (%s)' % val
    else:
        sql += ' and dt_stat=%d' % stat
        
    if bins != None and len(bins):
        val = ' or '.join(map(lambda x: 'dt_bin=' + str(x), bins))
        sql += ' and (%s)' % val

    if samples != None and len(samples):
        val = ' or '.join(map(lambda x: 'dt_sample=' + str(x), samples))
        sql += ' and (%s)' % val

    sql += ' group by dt_sample'

    sql = 'select avg(data) from (%s) as tb' % sql

    cursor.execute(sql)
    results = map(lambda x: x[0], cursor.fetchall())
    return results[0]

# Name: sumAvgMultSamp
# Desc: given a run, and a list of stats, samples and bins iterate over
#       the list of stats calling sum average on the samples, and bins
#       for each returning an numarray of the values
def sumAvgMultSamp(run,stats,samples,bins):
    results = nm.zeros(len(stats), type='Float64')
    count = 0
    for stat in stats:
         results[count] = sumAvg(run,stat,samples,bins)
         count = count + 1
    return results

# Name: sumAvgMultRuns
# Desc: Given a list of runs, stats, samples and bins, iterate over the list
#       of runs calling sumAvgMultSamp each time and returning an array of
#       numarrays
def sumAvgMultRuns(runs,stats,samples,bins):
    results = []
    for run in runs:
         results.append(sumAvgMultSamp(run,stats,samples,bins))
    return results

# Name: csvArray
# Desc: Take an numarray and convert it to a csv list
def csvArray(ar):
    outputStr = str(ar[0])
    for element in ar[1:]:
        outputStr += ', ' + str(element)
    return outputStr

# Name: getStatName
# Desc: Given an id of a stat, return it's name for printing
def getStatName(stat):
    cursor.execute("select st_name from stats where st_id = " + str(stat))
    res = cursor.fetchone()
    return res[0]

# Name: getRunName
# Desc: Given an id of a run, return it's name for printing
def getRunName(run):
    cursor.execute("select rn_name from runs where rn_id = " + str(run))
    res = cursor.fetchone()
    return res[0]

# Name: csvMatrix
# Desc: Produce CSV data for importing into a spread sheet given an
#       array of numarrays (what sumAvgMultRuns returns) the list of
#       stats that were used (in the same order), and a list of runs
#       also in the same order
def csvMatrix(ar, runs, stats):
   statsNames = ' '
   for stat in stats:
      statsNames += ', ' + getStatName(stat)
   runNames = []
   for run in runs:
      runNames.append(getRunName(run))

   print statsNames
   for runnum in range(0,len(ar)):
       print runNames[runnum] + ', '  + csvArray(ar[runnum])

# Name: bytesPerTick
# Desc: calculate the bytes per tick of various sets of bins for a given run
#       return an array of kernel, interrup, user, and idle bytesPerTick
def bytesPerTickBin(run,samples=None):
    bytes_stat = getStatFromName('server.tlaser.etherdev.rxBytes')
    ticks_stat = getStatFromName('server.full_cpu.numTicks')

    totalNumTicks = sumAvg(run,ticks_stat,samples)
    
    kernel = sumAvg(run,bytes_stat,samples,getNotBinsFromName("(es_intr|User stats|idle_thread)"))
    intr = sumAvg(run,bytes_stat,samples,getBinsFromName("es_intr"))
    user = sumAvg(run,bytes_stat,samples,getBinsFromName("User stats"))
    idle = sumAvg(run,bytes_stat,samples,getBinsFromName("idle_thread"))

    kernel_bpt = kernel/totalNumTicks
    intr_bpt = intr/totalNumTicks
    user_bpt = user/totalNumTicks
    idle_bpt = idle/totalNumTicks

    return [kernel_bpt, intr_bpt, user_bpt, idle_bpt]

def bytesPerTick(run,samples=None):
    bytes_stat = getStatFromName('server.tlaser.etherdev.rxBytes')
    ticks_stat = getStatFromName('server.full_cpu.numTicks')

    totalNumTicks = sumAvg(run,ticks_stat,samples)
    rxBytes = sumAvg(run,bytes_stat,samples)

    return rxBytes / totalNumTicks 

def ticksPerBin(run,samples=None):
    ticks_stat = getStatFromName('server.full_cpu.numTicks')

    kernel = sumAvg(run,ticks_stat,samples,getNotBinsFromName("(es_intr|User stats|idle_thread)"))
    intr = sumAvg(run,ticks_stat,samples,getBinsFromName("es_intr"))
    user = sumAvg(run,ticks_stat,samples,getBinsFromName("User stats"))
    idle = sumAvg(run,ticks_stat,samples,getBinsFromName("idle_thread"))

    return [kernel, intr, user, idle]

def overallMisses(run,samples=None):
    miss_stats = getStatFromName(r'server\.L2.*_mshr_misses')

    kernel = sumAvg(run,miss_stats,samples,getNotBinsFromName("(es_intr|User stats|idle_thread)"))
    intr = sumAvg(run,miss_stats,samples,getBinsFromName("es_intr"))
    user = sumAvg(run,miss_stats,samples,getBinsFromName("User stats"))
    idle = sumAvg(run,miss_stats,samples,getBinsFromName("idle_thread"))

    return [kernel, intr, user, idle]

def rxBytes(run,samples=None):
    stat = getStatFromName('server.tlaser.etherdev.rxBytes')

    kernel = sumAvg(run,stat,samples,getNotBinsFromName("(es_intr|User stats|idle_thread)"))
    intr = sumAvg(run,stat,samples,getBinsFromName("es_intr"))
    user = sumAvg(run,stat,samples,getBinsFromName("User stats"))
    idle = sumAvg(run,stat,samples,getBinsFromName("idle_thread"))

    return [kernel, intr, user, idle]

def rxPackets(run,samples=None):
    stat = getStatFromName('server.tlaser.etherdev.rxPackets')

    kernel = sumAvg(run,stat,samples,getNotBinsFromName("(es_intr|User stats|idle_thread)"))
    intr = sumAvg(run,stat,samples,getBinsFromName("es_intr"))
    user = sumAvg(run,stat,samples,getBinsFromName("User stats"))
    idle = sumAvg(run,stat,samples,getBinsFromName("idle_thread"))

    return [kernel, intr, user, idle]

def usage():
    print '''
Usage: %s [-d <db> ] [-h <host>] [-p] [-u <username>] <command> [command args]
''' % sys.argv[0]
    sys.exit(1)

def getopts(list, flags):
    import getopt
    try:
        opts, args = getopt.getopt(list, flags)
    except getopt.GetoptError:
        usage()

    return opts, args

if __name__ == '__main__':
    opts, args = getopts(sys.argv[1:], '-d:h:pu:')
    for o,a in opts:
        if o == '-d':
            dbConnect.db = a
        if o == '-h':
            dbConnect.host = a
        if o == '-p':
            dbConnect.passwd = getpass.getpass()
        if o == '-u':
            dbConnect.user = a

    if len(args) == 0:
        usage()

    dbConnect()

    if args[0] == 'stats':
        if len(args) == 1:
            listStats()
        elif len(args) == 2:
            listStats(args[1])
        else:
            usage()
    elif args[0] == 'runs':
        user = None
        opts, args = getopts(args[1:], '-u')
        if len(args):
            usage();
        for o,a in opts:
            if o == '-u':
                user = a
        listRuns(user)
    elif args[0] == 'bins':
        if len(args) == 1:
            listBins()
        elif len(args) == 2:
            listBins(args[1])
        else:
            usage()
    elif args[0] == 'samples':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            listSamples(run)
    elif args[0] == 'bpt' or args[0] == 'tpb':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            name = getRunName(run)
            value = bytesPerTick(run)
            if args[0] == 'bpt':
                print '%s, %f' % (name, 8 * value)
            else:
                if value != 0.0:
                    print '%s, %f' % (name, 1 / (8 * value))
                else:
                    print '%s, 0.0' % name
    elif args[0] == 'bptb' or args[0] == 'tpbb':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            name = getRunName(run)
            values = bytesPerTickBin(run)
            strings = []
            if args[0] == 'bptb':
                for value in values:
                    strings.append('%f' % (8 * value))
            else:
                for value in values:
                    if value != 0.0:
                        strings.append('%f' % (1 / (8 * value)))
                    else:
                        strings.append('0.0')

            print '%s: %s' % (name, ', '.join(strings))
    elif args[0] == 'ticks':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            name = getRunName(run)
            values = ticksPerBin(run)
            total = reduce(lambda x,y:x+y, values)
            strings = []
            for value in values:
                strings.append('%f' % (value / total))

            print '%s, %s' % (name, ', '.join(strings))
    elif args[0] == 'misses':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            name = getRunName(run)
            values = overallMisses(run)
            strings = []
            for value in values:
                strings.append('%f' % value)

            print '%s, %s' % (name, ', '.join(strings))
    elif args[0] == 'bytes':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            name = getRunName(run)
            values = rxBytes(run)
            strings = []
            for value in values:
                strings.append('%f' % value)

            print '%s, %s' % (name, ', '.join(strings))
    elif args[0] == 'packets':
        if len(args) == 1:
            runs = getAllRuns()
        elif len(args) == 2:
            runs = getRunFromName(args[1])
        else:
            usage()
        for run in runs:
            name = getRunName(run)
            values = rxPackets(run)
            strings = []
            for value in values:
                strings.append('%f' % value)

            print '%s, %s' % (name, ', '.join(strings))
    else:
        usage()


#    if len(args) > 0:
#        run = getRunFromName(args[0])
#        bytesPerTick(run[0])
