#!/usr/bin/perl 
# $Id: s.old_bkdwn.pl 1.32 04/02/02 18:14:10-08:00 stever@isabel.reinhardt.house $

use Getopt::Std;
require "$ENV{HOME}/abrevs.pl";

$debug = 0;

getopts('shdal:tex');


if ($#ARGV < 0) {
    print "\t$0 [-a][-t][-l <width>]\n";
    print "\t\t -a         : list ALL categories\n";
    print "\t\t -t         : don't print column totals\n";
    print "\t\t -e         : print EPR values\n";
    print "\t\t -d         : don't print divider prior to *.1.out files\n";
    print "\t\t -h         : don't print headers every 3 sections\n";
    print "\t\t -s         : print sums\n";
    print "\t\t -l <width> : set label field width to <width>\n";
    print "\t\t -x         : output suitable for import to Excel\n";
    print "\n\n";
}



$label_width = defined($opt_l) ? $opt_l : 5;
$prec = $label_width - 2;
$label_fmt = "$label_width.$prec";
print "Label format is \"$label_fmt\"\n" if $debug;


if ($opt_x) {
    $separator = ",";
}
else {
    $separator = " ";
}


##########################################################################
#
# OTHER is due to:
#
##########################################################################




#
# Define the categories to print.
#
#@categories = qw(IPC MisSp F:BW D:BW IQ I:BW QD:FU QD:M Q:Mem Q:FU Q:SB D:Pol F:Pol
#		 Polcy ICach Broke);
@categories = qw(IPC MisSp D:BW IQ I:BW QD:FU QD:M Q:Mem Q:FU Q:SB D:Pol ICach Broke);







#
# Define the mapping from causes generated by the simulator to the categories.
#
%cause_categories =
(
 'None' => 'None',

 'Bandwidth' => 'F:BW',



 # IQ wierdness
 'QFull:LSQ_full:IQ' => 'Other',
 'QFull:IQ_full:IQ'  => 'Other',

 'QFull:IQ_full:IssueBW'  => 'I:BW',

 'QFull:LSQ_full:IssueBW' => 'I:BW',

 # Redundant threading stuff
 'QFull:IQ_full:Deps:lvq-port'  => 'Red',
 'QFull:LSQ_full:Deps:lvq-port' => 'Red',
 'QFull:IQ_full:FU:lvq-port'    => 'Red',
 'QFull:LSQ_full:FU:lvq-port'   => 'Red',
 'QFull:ROB_full:FU:lvq-port'   => 'Red',


 'QFull:IQ_full:Deps:rd-port'  => 'QD:M',
 'QFull:LSQ_full:Deps:rd-port' => 'QD:M',

 'QFull:ROB_full:CommitBW' => 'C:BW',

 'QFull:ROB_full:StoreBuf'    => 'Q:SB',

 # Redundant threading stuff
 'QFull:ROB_full:RedRegCheck' => 'Red',
 'QFull:ROB_full:RedPredBuf'  => 'Red',
 'QFull:ROB_full:RedLVQ'      => 'Red',

 'QFull:IREG_full'  => 'Regs',
 'QFull:FPREG_full' => 'Regs',

 'QFull:No_Inst'    => 'Other',
 'QFull:ROB_cap'    => 'Polcy',
 'QFull:IQ_cap'     => 'Polcy',
 'QFull:BW'         => 'D:BW',

 'QFull:Policy'     => 'D:Pol',

 'BrLim'            => 'BrLim',

 'InvPC'            => 'MisSp',
 'BTB_Miss'         => 'MisSp',
 'BrRecover'        => 'MisSp',

 'Sync'             => 'Sync',
 'LowConf'          => 'LoCnf',
 'Policy'           => 'F:Pol',

 # Redundant threading stuff
 'RedIQRsrv'        => 'Red',
 'RedLSQRsrv'       => 'Red',
 'RedPredBuf'       => 'Red',

 'Unknown'          => 'Unk',

 'Zero_Prio' => 'F:Pol',

 #
 # Something is probably broken is these are non-zero
 #
 'QFull:IQ_full:Deps:none'     => 'Broke',
 'QFull:LSQ_full:Deps:none'    => 'Broke',
 'QFull:IQ_full:Deps:wr-port'  => 'Broke',
 'QFull:LSQ_full:Deps:wr-port' => 'Broke',
 'QFull:IQ_full:FU:none'       => 'Broke',
 'QFull:LSQ_full:FU:none'      => 'Broke',
 'QFull:ROB_full:FU:none'      => 'Broke',
 'QFull:ROB_full:IQ_Empty'     => 'Broke',
 'QFull:ROB_full:ROB_Empty'    => 'Broke',

);
 

sub mergeCauses
{
    my $cause = shift;
    my $cat;

    #
    # look up in %cause_categories hash first...
    # (if nothing there, then we check the matching below)
    #
    if ($cat = $cause_categories{$cause}) {
	return $cat;
    }

    #
    #  A couple more broken values
    #
    elsif ($cause =~ /broken/i) {
	return 'Broke';
    }
    elsif ($cause =~ /Cache:Hit\s*$/) {
	return 'Broke';
    }
    elsif ($cause =~ /Cache:\(null\)\s*$/) {
	return 'Broke';
    }
    elsif ($cause =~ /^QFull:(IQ|LSQ|ROB)_full:DCache:NotIssued/) {
	return 'Broke';
    }
    elsif ($cause =~ /^QFull:(IQ|LSQ)_full:NoInsts/) {
#	return 'Broke';
       return 'IQ';
    }


    #
    #  Easy to blame stuff on the I-Cache
    #
    elsif ($cause =~ /^ICache/) {
	return 'ICach';
    }

    #
    #  If we're out of read- or write-ports
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ|ROB)_full:(FU:(rd|wr)-port)/) {
	return 'Q:Mem';
    }

    #
    #  Any other FU problems
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ|ROB)_full:FU/) {
	return 'Q:FU';
    }

    #
    #  We've already filtered out deps on memory ops...
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ)_full:Deps/) {
	return 'QD:FU';
    }

    #
    #  Queue filled due to D-Cache problem
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ|ROB)_full:DCache/) {
	return 'Q:Mem';
    }

    #
    #  An instruction has been in the queue long enough to issue
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ)_full:TooYoung/) {
	return 'Q:Age';
    }

    #
    #  An instruction wasn't issued due to inorder restrictions
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ)_full:IssueInorder/) {
	return 'InOrd';
    }

    #
    #  Support for older output file formats
    #
    elsif ($cause =~ /^QFull:(IQ|LSQ)_full:Dep/) {
	return 'QD:FU';
    }
    elsif ($cause =~ /^QFull:ROB_full:Data/) {
	return 'QD:Mem';
    }
    elsif ($cause =~ /^QFull:(IQ|LSQ)_full:IssueMem/) {
	return 'QD:Mem';
    }

    #
    #  If we can't figure out how to categorize this...
    #
    else {
	die "No category for $cause";
    }
}


###########################################################################
#
#  Start of execution...
#
#
###########################################################################


#
# Would be nice to grab this from the data file
#
$fetch_width = 8;

$file_count=0;


#
# Here we go
#
$max_fn_length = 0;
foreach $file (@ARGV) {
    my $f;
    $f = &chop_fn( $file );
    if (length($f) > $max_fn_length) {
        $max_fn_length = length($f);
    }
}
$max_fn_length += 2;

$section = 0;
$num_dots = doHeader();
foreach $file (@ARGV) {

    undef(@floss);       # what comes out of read-stats
    undef(@fetch_loss);  # generated summary of @floss

    $need_cr = 0;



    undef $SIM_cycle;

    #  Reall ALL the stats from the file, creating variables in THIS package
    if (read_stats($file)) {
	print " -- Couldn't read stats from $file\n";
	next;
    }


    # number of threads is determined by looking at size of any array stat
    $num_threads = scalar(@floss);

    #
    #  Translation for older output file formats
    #
    if (!defined $SIM_cycle) {
	my $debug = 0;

	print "OLD output format\n  sim_cycle = $sim_cycle\n" if $debug;
	$SIM_cycle = $sim_cycle;

	if ($num_threads == 1) {
	    $COM_IPC = $IPC[0];
	    $FETCH_rate = $fetch_rate;
	    print "  IPC is $COM_IPC\n  fetch_rate is $FETCH_rate\n" if $debug;
	}
	else {
	    for (my $i=0; $i<$num_threads; ++$i) {
		$COM_IPC[$i] = $IPC[$i];
		$FETCH_rate[$i] = $fetch_rate[$i];
		print "  IPC is $COM_IPC[$i]\n  fetch_rate is $FETCH_rate[$i]\n" if $debug;
	    }
	}
    }

    # Grab list of causes out of any array
    @causes = keys %{$floss[0]};

    for ($i = 0; $i < $num_threads; ++$i) {

	foreach $cause (@causes) {

            #  We count instructions (slots) so we now need to divide 
            #  by cycles...	
	    $fetch_loss[$i]{mergeCauses($cause)} += 
              ($floss[$i]{$cause} / $SIM_cycle);
	}

	# Now account for actual executed instructions: useful & otherwise
	if ($num_threads > 1) {
	    $fetch_loss[$i]{'IPC'} = $COM_IPC[$i];
	    $fetch_loss[$i]{'MisSp'} = ($FETCH_rate[$i] - $COM_IPC[$i]);
	}
	else {
	    $fetch_loss[$i]{'IPC'} = $COM_IPC;
	    $fetch_loss[$i]{'MisSp'} = ($FETCH_rate - $COM_IPC);
	}
    }

    # '-a' option uses all categories (not just those
    # given in the @categories assignment above).
    if ($opt_a) {
	@categories = grep { $_ ne 'Other' } keys %{$fetch_loss[0]};
    }

    print "Using categories: @categories\n" if $debug;

    if ($opt_e) {
        @eprs = get_epr_values($file, "EPR");
    }


    if ($file =~ /\.[1a]\.out$/) {
	++$section;

	if ($section != 1) {
	    if (!$opt_d) {
		for (my $i=0; $i<$num_dots; ++$i) {
		    print ".";
		}
		$need_cr = 1;
	    }

	    if ( (($section % 3)==1) && !$opt_h) {
		if ($need_cr) {
		    $need_cr = 0;
		    print "\n";
		}
		
		doHeader();
	    }
	}
    }

    print "\n";


    $file = &chop_fn( $file );
    $fn_length = length $file;



    $sum = 0;

    #
    #  Zero out the cumulative sums
    #
    foreach $cat ( @categories ) {
	$csum{ $cat } = 0;
    }
    $csum{"Q:Tot"} = 0;
    $csum{"Other"} = 0;

    #
    #  Check to see if we actually have data here...
    #
    for ($i=0; $i<$num_threads; $i++) {
        $sum += $fetch_loss[$i]{'IPC'};
    }
    if ( $sum == 0 ) {
        printf("%-${max_fn_length}s No Results for this file\n\n","$file:");
        next;
    }
    else {
        $file_count += 1;
    }


    $sum = 0;

    #
    #  Loop through the threads
    #
    for ($i = 0; $i < $num_threads; ++$i) {

	$thread_sum = 0;

        #  Zero the queue-full sum for this thread
        $qf_sum = 0;

        if ($opt_x) {
            if ($i == 0) {
                # Print the filename before the first thread
                print "$file, ";
            }
            else {
                # skip filename for succeeding threads
                printf(" ,");
            }
        }
        else {
            if ($i == 0) {
                # Print the filename before the first thread
                printf("%-${max_fn_length}s", $file);
            }
            else {
                # Just print spaces after the first thread
                printf("%-${max_fn_length}s", " ");
            }
        }


        #
        #  Print the values in the specified categories
        #
	foreach $cat ( @categories ) {
	    $val = $fetch_loss[$i]{$cat};

            if ($opt_x) {
                print "$val, ";
            }
            else {
                printf(" %${label_fmt}f", $val);
            }
	    $sum += $val;
	    $thread_sum += $val;
	    $csum{ $cat } += $val;

            if ($cat =~ /^Q/) {
                $qf_sum += $val;
            }
	}

        #
        #  Print the sum of the QF categories
        #
        if ($opt_x) {
            print "$qf_sum, ";
        }
        else {
            printf(" %${label_fmt}f", $qf_sum);
        }
        $csum{"Q:Tot"} += $qf_sum;

	#
	#  optionally print the thread sums
	#
	if ($opt_s) {
            if ($opt_x) {
                print "$thread_sum, ";
            }
            else {
                printf(" %${label_fmt}f", $thread_sum);
            }
	}


        #
        # Only after the last thread, Print the OTHER category
        #
        if ($i == $num_threads-1) {
            # the "Other" categories are not
            # really per-thread, see sim-smt.c
            $val = 0;
            for ($j=0; $j<$num_threads; $j++) {
                $val += $fetch_loss[$j]{'Other'};
            }

            if ($opt_x) {
                print "$val, ";
            }
            else {
                printf(" %${label_fmt}f", $val);
            }
            $sum += $val;

	    if (0) {
		#
		#  Print the SUM TOTAL IPC value
		#
		if ($opt_x) {
		    print "$sum, ";
		}
		else {
		    printf(" %${label_fmt}f", $sum);
		}
	    }
        }
        else {
            if ($opt_x) {
                print " , ,";
            }
            else {
                printf("            ");
            }
        }


        #
        #  Print the EPR values...
        #
        if ( $opt_e ) {
            if ($opt_x) {
                print "$eprs[$i]\n";
            }
            else {
                printf(" %${label_fmt}f\n", $eprs[$i]);
            }
        }
        else {
            print "\n";
        }

    }

    #
    #  After we've done all the threads, total this workload
    #
    if (! $opt_t) {

        #
        #  Do the calculations...
        #

	foreach $cat (@categories) {
	    $bigsum{$cat} += $csum{$cat};
	    $sumcount += 1;
	}
        $bigsum{"Q:Tot"} += $csum{"Q:Tot"};

        if ($opt_e) {
            $bigsum{"EPR"} += $eprs[$num_threads];
        }

	#
	#  Print the per-file totals
	#
	if ($num_threads > 1) {
	  #  Space out to the column start
	  if ($opt_x) {
            print "total:,";
	  }
	  else {
            printf("%-${max_fn_length}s", 'total:'); 
	  }

	  foreach $cat (@categories) {
            if ($opt_x) {
	      print "$csum{$cat}, ";
            }
            else {
	      printf(" %${label_fmt}f", $csum{$cat});
            }
	  }

	  if ($opt_x) {
            print "$csum{'Q:Tot'}, ";
	  }
	  else {
            printf(" %${label_fmt}f", $csum{"Q:Tot"});
	  }

	  if ($opt_e) {
            if ($opt_x) {
                print " , , $eprs[$num_threads]";
            }
            else {
                printf("             %${label_fmt}f", $eprs[$num_threads]);
            }
	  }
	  print "\n";
	}
    }

    if (($fetch_width * 0.95) > $sum) {
	print "Warning: Sum of displayed values ($sum) is less than 95% of fetch width!\n";
    }

}
printf("\n");


#
#  Add column Averages after the last file...
#
if (! $opt_t and $file_count) {

    if ($opt_x) {
        my $temp;

        print "averages:,";

        foreach $cat (@categories) {
            $temp = $bigsum{$cat}/$file_count;
            print "$temp, ";
        }

        $temp = $bigsum{"Q:Tot"}/$file_count;
        print "$temp";

        if ($opt_e) {
            $temp = $bigsum{"EPR"}/$file_count;
            print", , , $temp";
        }
    }
    else {
        printf("%-${max_fn_length}s", 'averages:');

        foreach $cat (@categories) {
            printf(" %${label_fmt}f", ($bigsum{$cat}/$file_count));
        }
        printf(" %${label_fmt}f", ($bigsum{"Q:Tot"}/$file_count));

        if ($opt_e) {
            printf("             %${label_fmt}f", ($bigsum{"EPR"}/$file_count));
        }
    }

    print "\n";
}

exit;


###########################################################################


sub chop_fn {
    my ($file) = @_;

    $file =~ s?/run.err??;
    $file =~ s?-ref/cp\d/?-?g;
    $file =~ s/-amptjp//g;
    $file =~ s/-9stone21//g;
    $file =~ s/-primes//g;
    $file =~ s/-penguin//g;

    #  Removing path information
    $file =~ /^.*?([^\/]+)$/;
    $file = $1;

    $file =~ s/\.out$//;

    return $file;
}


#
# Read in the stats file.  Regular scalar statistics are read into the variable
# of the same name (e.g., $SIM_cycle).  Scalar stats that end in _<digit> are
# assumed to be per-thread and are read into an array (e.g., IPC_0 --> $IPC[0]).
# Distributions are read into hashes; if the name of the distribution ends in
# _<digit> an array of hashes is used.  The fetch_loss_counters dist is handled
# specially (since it's not a standard SimpleScalar distribution).
#
#  Also note that the colon character in a statistic or distribution name is replace
#  with an underscore for the variable name.
#
sub read_stats {
    my $file = shift;

    my $debug = 0;

    open(FILE, "<$file") or die "Can't open $file: $!";

    while (<FILE>)
    {
	s/^cpu.//;

	if (($stat, $val) = /^([\w\.\:]+)\s+(\d+\.?\d*)\s+\#.*/)
	{
	    $stat =~ s/\./_/;
	    $stat =~ s/_(\d)$/[\1]/;
	    $stat =~ s/:/_/;
	    print "Adding $stat = $val\n" if $debug;
	    eval "\$$stat = $val";
	}
	elsif (($dist) = /^([\w\.\:]+).start_dist/)
	{
	    $dist =~ s/_(\d)$/[\1]/;
	    $dist =~ s/:/_/;
	    while (<FILE>)
	    {
		last if /\.end_dist/;
		next if /\.max_value/;
		next if /\.min_value/;
		if (! /\s*([\w\/\(\)-]+(\s[\w\/]+)?)\s+(\d+)/) {
		    print "Can't get dist value for $dist! ($_)\n";
		    return 1;
		}
		print "Adding $dist\{$1\} = $3\n" if $debug;
		eval "\$$dist\{\'$1\'\} = $3";
	    }
        }
	elsif (/^fetch_loss_counters_(\d+).start/)
	{
	    $thread = $1;
	    print "Found start of FLoss counters for thread $thread\n" if $debug;

	    while (<FILE>)
	    {
		last if (/^fetch_loss_counters_(\d+).end/);

		($label, $floss_count) = split;
		$floss[$thread]{$label} = $floss_count;

		print "  FLoss: $label = $floss_count\n" if $debug;
	    }
	    print "Done with FLoss counters for thread $thread\n" if $debug;
	}
    }

    close(FILE);

    return 0;
}

#
# Print the header.
#
sub doHeader
{
    my $len = 0;

    if ($opt_x) {
        print "Benchmark, ";

        foreach my $cat ( @categories ) {
            print "$cat, ";
        }

        print "Q:Tot, ";

	if ($opt_s) {
	    print "Sum, ";
	}

	print "Other, ";

        if ($opt_e) {
            print "EPR";
        }
    }
    else {
        printf("%-${max_fn_length}s", 'Benchmark');
	$len = $max_fn_length;

        foreach my $cat ( @categories ) {
            printf(" %${label_width}s", $cat);
	    $len += ($label_width + 1);
        }

        printf(" %${label_width}s", 'Q:Tot');
	$len += (1 + $label_width);

	if ($opt_s) {
	    print "   Sum";
	    $len += 6;
	}

        printf(" %${label_width}s", 'Other');
	$len += (1 + $label_width);

        if ($opt_e) {
            print "   EPR";
	    $len += 6;
        }
    }

    print "\n";
    return $len;
}



my %base_ipc = {};
my %base_msipc = {};

sub get_epr_values {

    my $debug = 0;

    my ($file, $mode, $baseline_file) = @_;
    my $line, $num_threads=0, @thread_name;
    my $stat;

    my @rval;
    my $epr_total = 0;
    my $ipc_total = 0;


    if ($debug) {
	print "get_epr_values() called with file=$file, mode=$mode, baseline_file=$baseline_file\n";
    }

    if ($baseline_file eq "") {
	$baseline_file = "/n/dtw/z/sraasch/spec2000/pbs/results/baseline.ipc";
    }


    #
    #  Figure out how many threads in this file...
    #  Along the way, grab the benchark name...
    #
    $num_threads = 0;
    open IN, "<$file" or die "Can't open $file for reading!";
    while ( $line = <IN> ) {
        chomp $line;

        if ($line =~ /^workload=(.+)$/) {
	    @a = split / /, $1;
	    foreach $b (@a) {
		$c = $b;
		$c =~ s/cpu.//;

		print "Looking up abbreviation for \"$c\"...\n" if $debug;
		$thread_name[$num_threads] = $abrev{$c};

		print STDERR "found $b ($thread_name[$num_threads]) as thread #$num_threads\n" if $debug;
		$num_threads++;
	    }
	    last;
        }
        last if ($line =~ /simulation started/);
    }
    close IN;

    if ($mode eq "NAME") {
        return @thread_name;
        exit;
    }

    print "Looking for $num_threads threads...\n" if $debug;

    #
    #  Just do this once...
    #
    @k = keys %base_ipc;
    if ($base_ipc{$k[0]} == 0) {
	print "Getting base IPC data... \n" if $debug;
        get_base_ipc_data(\%base_ipc, \%base_msipc, $baseline_file);
    }

    #
    #  Get the individual IPC's for the threads
    #
    for ($i=0; $i<$num_threads; $i++) {
        $stat = `~sraasch/Public/bin/get -q IPC_$i -- $file`;
        chomp $stat;
        print STDERR "get returned '$stat' for IPC\n" if $debug;

        if ($stat =~ /(\d+\.\d+)\s+$/) {
            # Got a value
            $ipc = $1;

            print STDERR "IPC($thread_name[$i]) = $ipc\n" if $debug;

            if ($mode eq "EPR") {
                if ( $base_ipc{$thread_name[$i]} ) {
                    $epr = $ipc / $base_ipc{$thread_name[$i]};
                    $epr_total += $epr;
                    @rval = (@rval, $epr);
                }
                else {
                    @rval = (@rval, "N/A");
                }
            }
            elsif ($mode eq "IPC") {
                $ipc_total += $ipc;
                @rval = (@rval, $ipc);
            }
        }
        else {
              @rval = (@rval, "N/A");
        }
    }

    if ($mode eq "EPR") {
        @rval = (@rval, $epr_total);
    }
    elsif ($mode eq "IPC") {
        @rval = (@rval, $ipc_total);
    }

    print "Returning @rval\n" if $debug;

    return @rval;
}


#
#  Grab the base benchmark IPC's from the default location
#
sub get_base_ipc_data {

    my $debug = 0;

    my($ipc_hash_ref, $msipc_hash_ref, $baseline_file) = @_;

    open IN, "<$baseline_file" or
      die "Cannot open base IPC's for reading ($baseline_file) ";

    while ( $line = <IN> ) {
        chomp $line;

        next if $line =~ /^#/;

        if ($line =~ /\.(\w+)\.\d+\.out:\s+(\d+\.\d+)\s+(\d+\.\d+)/) {
            print STDERR "BASE: IPC{$1} = $2  MSIPC{$1} = $3\n" if $debug;
            $$ipc_hash_ref{$1} = $2;
            $$msipc_hash_ref{$1} = $3;
        }
        else {
            close IN;
            print STDERR "\"$line\"\n";
            die "syntax error in base IPC file";
        }
    }
    close IN;
}
