#!/bin/perl

use Getopt::Std;
use FileHandle;

#
#  Options:  -a <cycles>  fast-forward first trace
#            -b <cycles>  fast-forward second trace
#            -s           skip forward until things match up
#

getopts('a:b:s');

$debug = 0;

$sublen = 70;

$file[0] = shift;
$file[1] = shift;

#
#  Find the first cycle-marker in each file
#
for ($i=0; $i<2; ++$i) {
    $fh[$i] = new FileHandle "< $file[$i]";
    if (!defined $fh[$i]) {
	die "Can't open filehandle";
    }

    my $fileh = $fh[$i];

    my $done = 0;
    do {
	if (not $fileh->eof()) {
	    my $pos = $fileh->getpos();
	    $line = <$fileh>;
	    chomp $line;

	    print "$i: \"$line\"\n" if $debug;

	    if ($line =~ /^\@ (\d+)/) {
		$start[$i] = $1;
		$done = 1;
		$fileh->setpos($pos);
	    }
	}
	else {
	    print "Can't find initial cycle marker for file $file[$i]\n";
	    exit;
	}
    } while (!$done);
}

if ($debug) {
    print "Initial cycle for file 0 is $start[0]\n";
    print "Initial cycle for file 1 is $start[1]\n";
}

if ($opt_a or $opt_b) {
    if ($opt_s) {
	print "Cannot combine -s and -a or -b options!";
	exit;
    }

    if ($opt_a) {
	fast_forward($fh[0], $opt_a);
    }

    if ($opt_b) {
	fast_forward($fh[1], $opt_b);
    }
}
else {
    #
    #  We have the initial starting cycle numbers...
    #
    #  ... if they don't match, fast forward until they do
    #
    if (!$opt_s && ($start[0] != $start[1])) {
	if ($start[0] < $start[1]) {
	    $goal = $start[1];
	    $n = 0;
	}
	else {
	    $n = 1;
	    $goal = $start[0];
	}
	
	print "Fast-forwarding through file $n, looking for cycle $goal\n" 
	  if $debug;
	
	fast_forward($fh[$n], $goal);
	
    }
}


while (!$fh[0]->eof() && !$fh[1]->eof()) {
    print "getting chunk from file 0\n" if $debug;
    @chunk0 = get_cycle($fh[0]);
    print "getting chunk from file 1\n" if $debug;
    @chunk1 = get_cycle($fh[1]);

    $len = $#chunk0;
    if ($#chunk1 > $len) {
	$len = $#chunk1;
    }

    print "longest chunk index is $len\n" if $debug;

    for ($i=0; $i<=$len; ++$i) {
	my $a = $chunk0[$i];
	my $b = $chunk1[$i];
	
	my $debug2 = 0;

	if ($a ne $b) {
	    $marker = "|";
	    print "\'$a\'\n" if $debug2;
	    print "\'$b\'\n" if $debug2;
	}
	else {
	    $marker = " ";
	}

	print setlength($a, $sublen)." ".$marker." "."$b\n";
    }

    print "\n";
}

$fh[0]->close();
$fh[1]->close();

exit;


sub get_cycle {
    my ($h) = @_;
    my @chunk = ();

    my $debug = 0;

    my $done = 0;
    while(!$done) {
	$pos = $h->getpos();
	if ($line = <$h>) {
	    chomp $line;

	    print "get_cycle: \'$line\'\n" if $debug;

	    if (($line =~ /^\@/) && ($#chunk != -1)) {
		$done = 1;
		$h->setpos($pos);
	    }
	    else {
		@chunk = (@chunk, $line);
	    }
	}
	else {
	    print "EOF\n";
	    exit;
	}
    }

    return @chunk;
}


sub setlength {
    my ($s, $len) = @_;
    my $debug = 0;

    print "setting length of \'$s\' to $len\n" if $debug;

    if (length($s) > $len) {
	$r = substr $s, 0, $len;
    }
    elsif (length($s) < $len) {
	$r = $s;
	while (length($r) < $len) {
	    $r = "$r ";
	}
    }

    print "  result is \"$r\"\n" if $debug;

    return $r;
}

sub fast_forward {
    ($fh, $cycle) = @_;

    my $done = 0;
    do {
	if ($fh->eof()) {
	    print "Can't ff to cycle number\n";
	    exit;
	}

	my $pos = $fh->getpos();
	$line = <$fh>;
	chomp $line;
	
	if ($line =~ /^\@ (\d+)/) {
	    if ($1 == $cycle) {
		$done = 1;
		$fh->setpos($pos);
	    }
	}
    } while (!$done);
}
